cap log close
set more off
clear all

local date=subinstr(c(current_date), " ", " ", .) // set date 


** DEFINE HERE - FILE PATHS
** home directory 
global dir /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/
** log file path 
global log Programs/Dated_Logs/datapreparation.log
** location of the intermediate and output file directories 
global int /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/intermediate // set location for intermediate files
global out /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/final // set location for final data output
** Risk score file locations
global risk_scores /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/externally_generated/Risk_Scores/RiskScores_baseline.dta
global risk_scores_2006 /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/externally_generated/Risk_Scores/RiskScores06_baseline.dta
global MA_risk_scores /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/externally_generated/Risk_Scores/MA_riskScores_baseline.dta
** Medicare Part A and B utilization 
global partsab_util /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/externally_generated/MedicareA_B/utilization_tomerge.dta
** Medicare Part A and B death dates
global death_dates /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/externally_generated/MedicareA_B/death_dates_full.dta
** file paths for the raw beneficiary data. 
local file_path_2006 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/2006/ben20
local file_path_2007 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/2007/ben20
local file_path_2008 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/2008/ben20
local file_path_2009 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/bsf/2009/bsf
local file_path_2010 /disk/aging/medicare/data/20pct/bsf/2010/bsf
** number of subfiles each raw claims file is broken up into 
local num_files_2006 4
local num_files_2007 4
local num_files_2008 4
local num_files_2009 100
local num_files_2010 1
** file paths for raw claims 
local clm_2007 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/pde/2007/100files/stata/pde20_2007
local clm_2008 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/pde/2008/100files/stata/pde20_2008
local clm_2009 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/pde/2009/100files/pde20_2009
local clm_2010 /disk/aging/medicpd/20pct/pde/2010/pde2010
** file paths for raw plan
local plan /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/pln/2010/pln20_2010
local plan_2010 /disk/aging/medicpd/20pct/pln/2010/pln2010


cd ${dir}
log using ${log}_`date', replace 

** set the switches for what parts of the dataset generating we want to do 
scalar beneficiary = 1 // clean and append beneficiary files if == 1
scalar claim = 1 // clean and append claim data if == 1
scalar plan = 1  // clean plan data if plan == 1
scalar merge = 1  // merge beneficiary, claim, and plan data if == 1
scalar append = 1  // join all years if == 1
scalar baseline_restrictions = 1  // generate baseline sample
scalar wide = 1  // generate year-to-year dataset

* seed for random function 
set seed 72344

*****************************************************
*** STEP 1: Clean and append raw beneficiary data ***
*****************************************************
if beneficiary == 1 {

forv year = 2007/2010 {

* Data is broken up into a number of files per year
* Cycles through each of these files:
forvalues x = 1/`num_files_`year'' {

	* Load the beneficiary data, includes Parts A, B, C, and D so beneficiaries not enrolled 
	* in Part D are dropped in the next step

		if `year'~=2010 {
		*cycle through file parts
		use `file_path_`year''_`year'_`x', clear
		}

		if `year'==2010 {
		* Merge 2010 ab and d beneficiary data together so analogous to other years
		use `file_path_`year''ab`year', clear
		merge 1:1 bene_id using `file_path_`year''d`year'
		drop _m
		}

	
	* Generate a variable for months enrolled in Part D
	destring plncovmo, generate(partd) 
	label var partd "No. months part D coverage for beneficiary in year"

	* Drop beneficiaries not enrolled in Part D
	drop if partd == 0
	
	* Generate more useful indicators
	destring sex, replace
		drop if sex == 0
		label define sex_l 1 "male" 2 "female"
		label val sex sex_l
		gen female = sex==2
		label var female "female beneficiary"
	destring race, replace
		label define race_l 0 "unknown" 1 "white" 2 "black" 3 "other" 4 "asian" 5 "hispanic" 6 "north american native"
		label val race race_l
		gen white = race==1
		label var white "white beneficiary"
	gen death = 1
		replace death = 0 if death_dt == .
		label var death "Beneficiary died in plan year"

	* Generate a monthly plan ID- combo of contract and benefit package within contract
	foreach n in 01 02 03 04 05 06 07 08 09 10 11 12 {
		gen plan`n' = pbpid`n'+cntrct`n'
		label var plan`n' "contract and benefit package for month `n' in year"
		gen _cntrct`n' = substr(cntrct`n',1,1)
		label var _cntrct`n' "plan type"
	}
	
	* Generate a monthly switch dummy to identify beneficiaries who switch plans from one month to the next
	local switchmo "0102 0203 0304 0405 0506 0607 0708 0809 0910 1011 1112"
	foreach mo of local switchmo {
	 	local m=substr("`mo'",1,2)
	 	local n=substr("`mo'",3,4)
		gen switch`n' = (plan`m' != plan`n' & /// different plan id
			!(missing(plan`m') | _cntrct`m'=="0" | _cntrct`m'=="N") & /// enrolled in a plan last month
			!(missing(plan`n') | _cntrct`n'=="0" | _cntrct`n'=="N")) // enrolled in a plan this month
		label var switch`n' "switched contract or benefit package for month `n' in year"
	}	
	
	* Sum total number of switches per year
	gen numswitch = switch02 + switch03 + switch04 + switch05 + switch06 + switch07 + switch08 + switch09 + switch10 + switch11 + switch12
	label var numswitch "number of contract or benefit package switches during year"
	* Generate indicators for enrollment in Parts A and B
	assert a_mo_cnt!=.
	assert b_mo_cnt!=.
	gen parta=a_mo_cnt>0 & a_mo_cnt!=.
	label var parta "beneficiary enrolled in part a during year"
	gen partb=b_mo_cnt>0 & b_mo_cnt!=.
	label var partb "beneficiary enrolled in part b during year"

	* Generate a variables for enrollment and cost sharing- give each month enrolled value of month, and then take min to get enrollment month
	local i= 1
	foreach y in "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
		replace cstshr`y'="14" if cstshr`y'=="XX"
		gen temp=real(cstshr`y')
		gen partd_mo_`i'=`i' if temp<=9 & temp>0
		drop temp
		local i=`i'+1
	}

	* Pull beneficiaries' birth and join months
	gen birthmonth=month(bene_dob)
	egen partd_joinmonth=rowmin(partd_mo*)
	drop partd_mo_*

	* Generate a dummy for whether a beneficiary receives any cost sharing
	gen no_costsharing = 1
		foreach n in 01 02 03 04 05 06 07 08 09 10 11 12 {
			replace no_costsharing = 0 if cstshr`n'!="09" & partd_joinmonth<=`n'
	}
	label var no_costsharing "no subsidy or cost sharing for any months while on part d"
	

	if `year'==2010 {
	*no strict flag on 2010 because sample is already strict
	gen strctflg="100"
	}

	keep bene_id bene_zip state_cd sex race white female age dual_mo death numswitch crdcovsw orec ///
	plan12 parta partb partd partd_joinmonth birthmonth bene_dob no_costsharing strctflg
		
	tempfile ben_`year'_`x'
	save `ben_`year'_`x''

} // end loop over multiple files withing year

* Append files to create a complete file for each year
use `ben_`year'_1', clear
	
	if `year'~=2010 {
		forvalues x = 2/`num_files_`year'' {
			append using `ben_`year'_`x'', force
		}
	}
	
save ${int}/benmerge`year', replace

keep bene_id partd_joinmonth
save ${int}/allage_joinmonth_`year', replace

} // end foreach

} // end beneficiary 

*****************************************************
****** STEP 2: Clean and append raw claim data ******
*****************************************************
if claim == 1 {

foreach year of numlist 2010/2007 {

	if `year'==2006|`year'==2007|`year'==2008 {
		forvalues x = 1/4 {
		forvalues y = 1/100 {	
			di "CLAIMS FILE YEAR: `year' X: `x' Y: `y'"
			* Load the claims subfile (400 in 2007 and 2008, 100 in 2009) 
			*if re-running this, need to change location to reflect where data is 
			use `clm_`year''_`x'_`y', clear
			* Run the claims preparation code
					* Generate unique plan identifier
					gen plan = plnpbprc + plncntrc  

					* Drop if claim is not covered
					drop if drcvstcd!="C"

					* Generating spending that occurs in each phase to estimate cost sharing
						* DEDUCTIBLE
						gen ded_totcost = 0
						gen ded_oopspend = 0
						replace ded_totcost = totalcst if bnftphas == "DD"
						replace ded_oopspend = ptpayamt if bnftphas == "DD"
						* PRE-ICL
						gen preicl_totcost = 0
						gen preicl_oopspend = 0
						replace preicl_totcost = totalcst if bnftphas == "PP" 
						replace preicl_oopspend = ptpayamt if bnftphas == "PP"
						* GAP 
						gen gap_totcost = 0
						gen gap_oopspend = 0
						replace gap_totcost = totalcst if bnftphas == "II" 
						replace gap_oopspend = ptpayamt if bnftphas == "II" 
						* CATASTROPHIC
						gen cat_totcost = 0
						gen cat_oopspend = 0 
						replace cat_totcost = totalcst if bnftphas == "CC"
						replace cat_oopspend = ptpayamt if bnftphas == "CC"

					* To obtain a total number of claims in the year
					gen count = 1 

					* Making sure no observations are missing service dates
					replace srvc_dt = td(01jan1960) if missing(srvc_dt)

					* Merge in external drug classification data
					local drug_class_list "chronic_drug fdb_generic maint hedis_ind gtc"
					merge m:1 prdsrvid using Analytic_Data/final/Drug_Classification/DrugClassifications.dta, keepusing(`drug_class_list') gen(_merge_drug)

					assert _merge_drug!=1
					drop if _merge_drug~=3

					* Merge in the month of the year the beneficiary joins Part D (from beneficiary file)
					merge m:1 bene_id using ${int}/allage_joinmonth_`year', keep(match master) gen(_merge_jm)
					gen partd_joinday = mdy(partd_joinmonth, 1, `year')

					* Generating total spending, date of last claim, and January spending for each drug type
						* Classifying claims
						
						gen clm_any = 1
						gen clm_acute = (chronic_drug==0)
						gen clm_chr = (chronic_drug==1)
						gen clm_gen = (fdb_generic==1)
						gen clm_brand = (fdb_generic==0)
						gen clm_maint = (maint==1)
						gen clm_nonmaint = (maint==0)
						gen clm_hedis = (hedis_ind==1)
						gen clm_nonhedis = (hedis_ind==0)
				
						
						* Generating monthly spending for select drug types, date of last claim
						foreach c in any acute chr gen brand maint nonmaint hedis nonhedis  {
							
							gen total_`c' = totalcst if clm_`c'
							replace total_`c' = 0 if total_`c'==.

							gen date_`c' = srvc_dt if clm_`c'
								replace date_`c'=td(01jan1960) if date_`c'==.

							foreach month of numlist 1/12 {
								gen m`month'_total_`c' = totalcst if month(srvc_dt)==`month' & clm_`c'==1
								replace m`month'_total_`c' = 0 if missing(m`month'_total_`c')
							}
						}

					*Generating number of claims per month, average fill size, under vs over 30 fill size
					foreach month of numlist 1/12 {
						gen m`month'_claims = 1 if month(srvc_dt)==`month' & clm_any==1
								replace m`month'_claims = 0 if missing(m`month'_claims)
						gen fillsize_m`month'=dayssply if month(srvc_dt)==`month'
						
						gen m`month'_l30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply<30
								replace m`month'_l30_claims = 0 if missing(m`month'_l30_claims)
						gen m`month'_e30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply==30
								replace m`month'_e30_claims = 0 if missing(m`month'_e30_claims)
						gen m`month'_g30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply>30
								replace m`month'_g30_claims = 0 if missing(m`month'_g30_claims)
					}


					*By week
					foreach week of numlist 1/4 48/52 {

						gen w`week'_total_any =  totalcst if week(srvc_dt)==`week' & clm_any==1
								replace w`week'_total_any = 0 if missing(w`week'_total_any)
						gen fillsize_w`week'=dayssply if week(srvc_dt)==`week'
								
					}

					 *Generating initial utilization variables for age 65 join month analysis
					gen initial_total_any = totalcst if month(srvc_dt)<(partd_joinmonth+3)
						replace initial_total_any = 0 if initial_total_any==.
					gen days_to_firstclaim_any = date_any-partd_joinday+1
						replace days_to_firstclaim_any = 92 if (days_to_firstclaim_any>92|days_to_firstclaim_any<1)

					/*
					* Generating total spending and date of last claim for each therapeutic class (GTC)
					foreach c in 2 11 14 20 23 32 35 38 41 56 59 62 65 68 71 80 86 89 {
						gen total_gtc`c' = totalcst if gtc==`c'
							replace total_gtc`c' = 0 if total_gtc`c'==.
						gen date_gtc`c' = srvc_dt if gtc==`c'
							replace date_gtc`c'=td(01jan1960) if date_gtc`c'==.
					}

					*/

					*Switching claims from brand to generic

						*number of claims for each month
						foreach month of numlist 1/12 {
							gen m`month'_claims_gen = 1 if month(srvc_dt)==`month' & clm_gen==1
									replace m`month'_claims_gen = 0 if missing(m`month'_claims_gen)
							gen m`month'_claims_brand = 1 if month(srvc_dt)==`month' & clm_brand==1
									replace m`month'_claims_brand = 0 if missing(m`month'_claims_brand)
						}
						
						gen early_brand_any = 1 if month(srvc_dt)<10 & clm_brand==1
						replace early_brand_any= 0 if missing(early_brand_any)

					foreach c in 2 3 5 8 9 11 14 17 18 20 23 26 29 32 35 38 41 44 47 50 53 56 59 62 65 68 71 72 74 77 80 83 86 89 92 94 95 97 98 99 {
						
						gen early_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)<10 & clm_brand==1
							replace early_brand_gtc_`c' = 0 if missing(early_brand_gtc_`c')
						
						gen jan_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==1 & clm_brand==1
							replace jan_brand_gtc_`c' = 0 if missing(jan_brand_gtc_`c')
						
						gen jan_gen_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==1 & clm_gen==1
							replace jan_gen_gtc_`c' = 0 if missing(jan_gen_gtc_`c')
						
						gen dec_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==12 & clm_brand==1
							replace dec_brand_gtc_`c' = 0 if missing(dec_brand_gtc_`c')
						
						gen dec_gen_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==12 & clm_gen==1
							replace dec_gen_gtc_`c' = 0 if missing(dec_gen_gtc_`c')
					}

					* Coding final phase numerically to ensure it collapses correctly
					gen finphas = 0
					replace finphas = 1 if bnftphas=="XX"
					replace finphas = 2 if bnftphas=="NA"
					replace finphas = 3 if bnftphas=="DD"
					replace finphas = 4 if bnftphas=="DP"
					replace finphas = 5 if bnftphas=="PP"
					replace finphas = 6 if bnftphas=="DI"
					replace finphas = 7 if bnftphas=="PI"
					replace finphas = 8 if bnftphas=="II"
					replace finphas = 9 if bnftphas=="DC"
					replace finphas = 10 if bnftphas=="PC"
					replace finphas = 11 if bnftphas=="IC"
					replace finphas = 12 if bnftphas=="CC"


					* Collapse the data to the beneficiary-plan-year level
					collapse ///
					(sum) count oopspend=ptpayamt totcost=totalcst total_* clm_* initial_* ///
						preicl_totcost preicl_oopspend gap_totcost gap_oopspend ded_totcost ded_oopspend ///
						cat_totcost cat_oopspend m1* m2* m3* m4* m5* m6* m7* m8* m9* lics_amt plro_amt othtroop ///
						w1* w2* w3* w4* w5* ///
					(mean) fillsize* ///
					(max) finalphase=finphas date_* early_brand* jan_brand* jan_gen* dec_brand* dec_gen* ///
					(min) days_*, ///
					by(plan bene_id) fast

					* Recoding final phase variable to string
					gen finphas = ""
					replace finphas = "XX" if finalphase==1
					replace finphas = "NA" if finalphase==2
					replace finphas = "DD" if finalphase==3
					replace finphas = "DP" if finalphase==4
					replace finphas = "PP" if finalphase==5
					replace finphas = "DI" if finalphase==6
					replace finphas = "PI" if finalphase==7
					replace finphas = "II" if finalphase==8
					replace finphas = "DC" if finalphase==9
					replace finphas = "PC" if finalphase==10
					replace finphas = "IC" if finalphase==11
					replace finphas = "CC" if finalphase==12

					format date_* %td 
			* Save intermediate collapsed claims file
			tempfile claims_`year'_`x'_`y'
			save `claims_`year'_`x'_`y'', replace
		} // end y
		} // end x

		* Append all the collapsed subfiles together to create one file for each year
		use `claims_`year'_1_1', clear
			drop if !missing(bene_id)
			forv x = 1/4 {
			forv y = 1/100 {
				append using `claims_`year'_`x'_`y''
			} // end y
			} // end x

		save ${int}/claimsmerge`year', replace

	}  // end y06-y08

	if `year'==2009 {
		forv y = 1/100 {
			di "CLAIMS FILE YEAR: `year' Y: `y'"
			* Load the claims subfile (400 in 2007 and 2008, 100 in 2009)
			*if re-running this, need to change location to reflect where data is 
			use `clm_2009'_`y', clear
			* Run the claims preparation code
					* Generate unique plan identifier
					gen plan = plnpbprc + plncntrc  

					* Drop if claim is not covered
					drop if drcvstcd!="C"

					* Generating spending that occurs in each phase to estimate cost sharing
						* DEDUCTIBLE
						gen ded_totcost = 0
						gen ded_oopspend = 0
						replace ded_totcost = totalcst if bnftphas == "DD"
						replace ded_oopspend = ptpayamt if bnftphas == "DD"
						* PRE-ICL
						gen preicl_totcost = 0
						gen preicl_oopspend = 0
						replace preicl_totcost = totalcst if bnftphas == "PP" 
						replace preicl_oopspend = ptpayamt if bnftphas == "PP"
						* GAP 
						gen gap_totcost = 0
						gen gap_oopspend = 0
						replace gap_totcost = totalcst if bnftphas == "II" 
						replace gap_oopspend = ptpayamt if bnftphas == "II" 
						* CATASTROPHIC
						gen cat_totcost = 0
						gen cat_oopspend = 0 
						replace cat_totcost = totalcst if bnftphas == "CC"
						replace cat_oopspend = ptpayamt if bnftphas == "CC"

					* To obtain a total number of claims in the year
					gen count = 1 

					* Making sure no observations are missing service dates
					replace srvc_dt = td(01jan1960) if missing(srvc_dt)

					* Merge in external drug classification data
					local drug_class_list "chronic_drug fdb_generic maint hedis_ind gtc"
					merge m:1 prdsrvid using Analytic_Data/final/Drug_Classification/DrugClassifications.dta, keepusing(`drug_class_list') gen(_merge_drug)

					assert _merge_drug!=1
					drop if _merge_drug~=3

					* Merge in the month of the year the beneficiary joins Part D (from beneficiary file)
					merge m:1 bene_id using ${int}/allage_joinmonth_`year', keep(match master) gen(_merge_jm)
					gen partd_joinday = mdy(partd_joinmonth, 1, `year')

					* Generating total spending, date of last claim, and January spending for each drug type
						* Classifying claims
						
						gen clm_any = 1
						gen clm_acute = (chronic_drug==0)
						gen clm_chr = (chronic_drug==1)
						gen clm_gen = (fdb_generic==1)
						gen clm_brand = (fdb_generic==0)
						gen clm_maint = (maint==1)
						gen clm_nonmaint = (maint==0)
						gen clm_hedis = (hedis_ind==1)
						gen clm_nonhedis = (hedis_ind==0)
				
						
						* Generating monthly spending for select drug types, date of last claim
						foreach c in any acute chr gen brand maint nonmaint hedis nonhedis  {
							
							gen total_`c' = totalcst if clm_`c'
							replace total_`c' = 0 if total_`c'==.

							gen date_`c' = srvc_dt if clm_`c'
								replace date_`c'=td(01jan1960) if date_`c'==.

							foreach month of numlist 1/12 {
								gen m`month'_total_`c' = totalcst if month(srvc_dt)==`month' & clm_`c'==1
								replace m`month'_total_`c' = 0 if missing(m`month'_total_`c')
							}
						}

					*Generating number of claims per month, average fill size, under vs over 30 fill size
					foreach month of numlist 1/12 {
						gen m`month'_claims = 1 if month(srvc_dt)==`month' & clm_any==1
								replace m`month'_claims = 0 if missing(m`month'_claims)
						gen fillsize_m`month'=dayssply if month(srvc_dt)==`month'
						
						gen m`month'_l30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply<30
								replace m`month'_l30_claims = 0 if missing(m`month'_l30_claims)
						gen m`month'_e30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply==30
								replace m`month'_e30_claims = 0 if missing(m`month'_e30_claims)
						gen m`month'_g30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply>30
								replace m`month'_g30_claims = 0 if missing(m`month'_g30_claims)
					}


					*By week
					foreach week of numlist 1/4 48/52 {

						gen w`week'_total_any =  totalcst if week(srvc_dt)==`week' & clm_any==1
								replace w`week'_total_any = 0 if missing(w`week'_total_any)
						gen fillsize_w`week'=dayssply if week(srvc_dt)==`week'
								
					}

					 *Generating initial utilization variables for age 65 join month analysis
					gen initial_total_any = totalcst if month(srvc_dt)<(partd_joinmonth+3)
						replace initial_total_any = 0 if initial_total_any==.
					gen days_to_firstclaim_any = date_any-partd_joinday+1
						replace days_to_firstclaim_any = 92 if (days_to_firstclaim_any>92|days_to_firstclaim_any<1)

					/*
					* Generating total spending and date of last claim for each therapeutic class (GTC)
					foreach c in 2 11 14 20 23 32 35 38 41 56 59 62 65 68 71 80 86 89 {
						gen total_gtc`c' = totalcst if gtc==`c'
							replace total_gtc`c' = 0 if total_gtc`c'==.
						gen date_gtc`c' = srvc_dt if gtc==`c'
							replace date_gtc`c'=td(01jan1960) if date_gtc`c'==.
					}

					*/

					*Switching claims from brand to generic

						*number of claims for each month
						foreach month of numlist 1/12 {
							gen m`month'_claims_gen = 1 if month(srvc_dt)==`month' & clm_gen==1
									replace m`month'_claims_gen = 0 if missing(m`month'_claims_gen)
							gen m`month'_claims_brand = 1 if month(srvc_dt)==`month' & clm_brand==1
									replace m`month'_claims_brand = 0 if missing(m`month'_claims_brand)
						}
						
						gen early_brand_any = 1 if month(srvc_dt)<10 & clm_brand==1
						replace early_brand_any= 0 if missing(early_brand_any)

					foreach c in 2 3 5 8 9 11 14 17 18 20 23 26 29 32 35 38 41 44 47 50 53 56 59 62 65 68 71 72 74 77 80 83 86 89 92 94 95 97 98 99 {
						
						gen early_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)<10 & clm_brand==1
							replace early_brand_gtc_`c' = 0 if missing(early_brand_gtc_`c')
						
						gen jan_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==1 & clm_brand==1
							replace jan_brand_gtc_`c' = 0 if missing(jan_brand_gtc_`c')
						
						gen jan_gen_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==1 & clm_gen==1
							replace jan_gen_gtc_`c' = 0 if missing(jan_gen_gtc_`c')
						
						gen dec_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==12 & clm_brand==1
							replace dec_brand_gtc_`c' = 0 if missing(dec_brand_gtc_`c')
						
						gen dec_gen_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==12 & clm_gen==1
							replace dec_gen_gtc_`c' = 0 if missing(dec_gen_gtc_`c')
					}

					* Coding final phase numerically to ensure it collapses correctly
					gen finphas = 0
					replace finphas = 1 if bnftphas=="XX"
					replace finphas = 2 if bnftphas=="NA"
					replace finphas = 3 if bnftphas=="DD"
					replace finphas = 4 if bnftphas=="DP"
					replace finphas = 5 if bnftphas=="PP"
					replace finphas = 6 if bnftphas=="DI"
					replace finphas = 7 if bnftphas=="PI"
					replace finphas = 8 if bnftphas=="II"
					replace finphas = 9 if bnftphas=="DC"
					replace finphas = 10 if bnftphas=="PC"
					replace finphas = 11 if bnftphas=="IC"
					replace finphas = 12 if bnftphas=="CC"


					* Collapse the data to the beneficiary-plan-year level
					collapse ///
					(sum) count oopspend=ptpayamt totcost=totalcst total_* clm_* initial_* ///
						preicl_totcost preicl_oopspend gap_totcost gap_oopspend ded_totcost ded_oopspend ///
						cat_totcost cat_oopspend m1* m2* m3* m4* m5* m6* m7* m8* m9* lics_amt plro_amt othtroop ///
						w1* w2* w3* w4* w5* ///
					(mean) fillsize* ///
					(max) finalphase=finphas date_* early_brand* jan_brand* jan_gen* dec_brand* dec_gen* ///
					(min) days_*, ///
					by(plan bene_id) fast

					* Recoding final phase variable to string
					gen finphas = ""
					replace finphas = "XX" if finalphase==1
					replace finphas = "NA" if finalphase==2
					replace finphas = "DD" if finalphase==3
					replace finphas = "DP" if finalphase==4
					replace finphas = "PP" if finalphase==5
					replace finphas = "DI" if finalphase==6
					replace finphas = "PI" if finalphase==7
					replace finphas = "II" if finalphase==8
					replace finphas = "DC" if finalphase==9
					replace finphas = "PC" if finalphase==10
					replace finphas = "IC" if finalphase==11
					replace finphas = "CC" if finalphase==12

					format date_* %td 
			
			
			
			* Save intermKresge 408ediate collapsed claims file
			tempfile claims_`year'_`y'
			save `claims_`year'_`y''
		} // end y
		
		* Append files to create a complete file for 2009
		use `claims_`year'_1', clear
			forv y = 2/100 {
				append using `claims_`year'_`y''
			} // end y
		
		save ${int}/claimsmerge`year', replace

	} // end y09
	
	if `year'==2010 {
		
		* Load the claims subfile (only 1 for 2010) and sort by beneficiary and claim id
		*if re-running this, need to change location to reflect where data is 
		local pde_varlist "bene_id pde_id plnpbprc plncntrc drcvstcd prdsrvid totalcst ptpayamt bnftphas srvc_dt lics_amt plro_amt othtroop dayssply"
		use `pde_varlist' using `clm_2010', clear
		sort bene_id pde_id, stable
		isid bene_id pde_id

		*split into 10 files
		egen person=group(bene_id)
		sum person
		local num_bene=r(max)
		gen partition=0
		foreach num of numlist .1(.1)1 {
			replace partition=(10*`num') if person>(`num_bene'*(`num'-.1)) & person<=(`num_bene'*(`num'))
		}
		drop person
		count
		di "sample"
		di r(N)
		tempfile partitioned_claims_`year'
		save `partitioned_claims_`year''

		forv y = 1/10 {
			di "CLAIMS FILE YEAR: `year' Y: `y'"
			* Load the claims subfile (400 in 2007 and 2008, 100 in 2009, 10 in 2010)
			use `partitioned_claims_`year'' if partition==`y', clear
			count
			di "sample"
			di r(N)
			* Run the claims preparation code
					* Generate unique plan identifier
					gen plan = plnpbprc + plncntrc  

					* Drop if claim is not covered
					drop if drcvstcd!="C"

					* Generating spending that occurs in each phase to estimate cost sharing
						* DEDUCTIBLE
						gen ded_totcost = 0
						gen ded_oopspend = 0
						replace ded_totcost = totalcst if bnftphas == "DD"
						replace ded_oopspend = ptpayamt if bnftphas == "DD"
						* PRE-ICL
						gen preicl_totcost = 0
						gen preicl_oopspend = 0
						replace preicl_totcost = totalcst if bnftphas == "PP" 
						replace preicl_oopspend = ptpayamt if bnftphas == "PP"
						* GAP 
						gen gap_totcost = 0
						gen gap_oopspend = 0
						replace gap_totcost = totalcst if bnftphas == "II" 
						replace gap_oopspend = ptpayamt if bnftphas == "II" 
						* CATASTROPHIC
						gen cat_totcost = 0
						gen cat_oopspend = 0 
						replace cat_totcost = totalcst if bnftphas == "CC"
						replace cat_oopspend = ptpayamt if bnftphas == "CC"

					* To obtain a total number of claims in the year
					gen count = 1 

					* Making sure no observations are missing service dates
					replace srvc_dt = td(01jan1960) if missing(srvc_dt)

					* Merge in external drug classification data
					local drug_class_list "chronic_drug fdb_generic maint hedis_ind gtc"
					merge m:1 prdsrvid using Analytic_Data/final/Drug_Classification/DrugClassifications.dta, keepusing(`drug_class_list') gen(_merge_drug)

					assert _merge_drug!=1
					drop if _merge_drug~=3

					* Merge in the month of the year the beneficiary joins Part D (from beneficiary file)
					merge m:1 bene_id using ${int}/allage_joinmonth_`year', keep(match master) gen(_merge_jm)
					gen partd_joinday = mdy(partd_joinmonth, 1, `year')

					* Generating total spending, date of last claim, and January spending for each drug type
						* Classifying claims
						
						gen clm_any = 1
						gen clm_acute = (chronic_drug==0)
						gen clm_chr = (chronic_drug==1)
						gen clm_gen = (fdb_generic==1)
						gen clm_brand = (fdb_generic==0)
						gen clm_maint = (maint==1)
						gen clm_nonmaint = (maint==0)
						gen clm_hedis = (hedis_ind==1)
						gen clm_nonhedis = (hedis_ind==0)
				
						
						* Generating monthly spending for select drug types, date of last claim
						foreach c in any acute chr gen brand maint nonmaint hedis nonhedis  {
							
							gen total_`c' = totalcst if clm_`c'
							replace total_`c' = 0 if total_`c'==.

							gen date_`c' = srvc_dt if clm_`c'
								replace date_`c'=td(01jan1960) if date_`c'==.

							foreach month of numlist 1/12 {
								gen m`month'_total_`c' = totalcst if month(srvc_dt)==`month' & clm_`c'==1
								replace m`month'_total_`c' = 0 if missing(m`month'_total_`c')
							}
						}

					*Generating number of claims per month, average fill size, under vs over 30 fill size
					foreach month of numlist 1/12 {
						gen m`month'_claims = 1 if month(srvc_dt)==`month' & clm_any==1
								replace m`month'_claims = 0 if missing(m`month'_claims)
						gen fillsize_m`month'=dayssply if month(srvc_dt)==`month'
						
						gen m`month'_l30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply<30
								replace m`month'_l30_claims = 0 if missing(m`month'_l30_claims)
						gen m`month'_e30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply==30
								replace m`month'_e30_claims = 0 if missing(m`month'_e30_claims)
						gen m`month'_g30_claims = 1 if month(srvc_dt)==`month' & clm_any==1 & dayssply>30
								replace m`month'_g30_claims = 0 if missing(m`month'_g30_claims)
					}


					*By week
					foreach week of numlist 1/4 48/52 {

						gen w`week'_total_any =  totalcst if week(srvc_dt)==`week' & clm_any==1
								replace w`week'_total_any = 0 if missing(w`week'_total_any)
						gen fillsize_w`week'=dayssply if week(srvc_dt)==`week'
								
					}

					 *Generating initial utilization variables for age 65 join month analysis
					gen initial_total_any = totalcst if month(srvc_dt)<(partd_joinmonth+3)
						replace initial_total_any = 0 if initial_total_any==.
					gen days_to_firstclaim_any = date_any-partd_joinday+1
						replace days_to_firstclaim_any = 92 if (days_to_firstclaim_any>92|days_to_firstclaim_any<1)

					/*
					* Generating total spending and date of last claim for each therapeutic class (GTC)
					foreach c in 2 11 14 20 23 32 35 38 41 56 59 62 65 68 71 80 86 89 {
						gen total_gtc`c' = totalcst if gtc==`c'
							replace total_gtc`c' = 0 if total_gtc`c'==.
						gen date_gtc`c' = srvc_dt if gtc==`c'
							replace date_gtc`c'=td(01jan1960) if date_gtc`c'==.
					}

					*/

					*Switching claims from brand to generic

						*number of claims for each month
						foreach month of numlist 1/12 {
							gen m`month'_claims_gen = 1 if month(srvc_dt)==`month' & clm_gen==1
									replace m`month'_claims_gen = 0 if missing(m`month'_claims_gen)
							gen m`month'_claims_brand = 1 if month(srvc_dt)==`month' & clm_brand==1
									replace m`month'_claims_brand = 0 if missing(m`month'_claims_brand)
						}
						
						gen early_brand_any = 1 if month(srvc_dt)<10 & clm_brand==1
						replace early_brand_any= 0 if missing(early_brand_any)

					foreach c in 2 3 5 8 9 11 14 17 18 20 23 26 29 32 35 38 41 44 47 50 53 56 59 62 65 68 71 72 74 77 80 83 86 89 92 94 95 97 98 99 {
						
						gen early_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)<10 & clm_brand==1
							replace early_brand_gtc_`c' = 0 if missing(early_brand_gtc_`c')
						
						gen jan_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==1 & clm_brand==1
							replace jan_brand_gtc_`c' = 0 if missing(jan_brand_gtc_`c')
						
						gen jan_gen_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==1 & clm_gen==1
							replace jan_gen_gtc_`c' = 0 if missing(jan_gen_gtc_`c')
						
						gen dec_brand_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==12 & clm_brand==1
							replace dec_brand_gtc_`c' = 0 if missing(dec_brand_gtc_`c')
						
						gen dec_gen_gtc_`c' = 1 if gtc==`c' & month(srvc_dt)==12 & clm_gen==1
							replace dec_gen_gtc_`c' = 0 if missing(dec_gen_gtc_`c')
					}

					* Coding final phase numerically to ensure it collapses correctly
					gen finphas = 0
					replace finphas = 1 if bnftphas=="XX"
					replace finphas = 2 if bnftphas=="NA"
					replace finphas = 3 if bnftphas=="DD"
					replace finphas = 4 if bnftphas=="DP"
					replace finphas = 5 if bnftphas=="PP"
					replace finphas = 6 if bnftphas=="DI"
					replace finphas = 7 if bnftphas=="PI"
					replace finphas = 8 if bnftphas=="II"
					replace finphas = 9 if bnftphas=="DC"
					replace finphas = 10 if bnftphas=="PC"
					replace finphas = 11 if bnftphas=="IC"
					replace finphas = 12 if bnftphas=="CC"


					* Collapse the data to the beneficiary-plan-year level
					collapse ///
					(sum) count oopspend=ptpayamt totcost=totalcst total_* clm_* initial_* ///
						preicl_totcost preicl_oopspend gap_totcost gap_oopspend ded_totcost ded_oopspend ///
						cat_totcost cat_oopspend m1* m2* m3* m4* m5* m6* m7* m8* m9* lics_amt plro_amt othtroop ///
						w1* w2* w3* w4* w5* ///
					(mean) fillsize* ///
					(max) finalphase=finphas date_* early_brand* jan_brand* jan_gen* dec_brand* dec_gen* ///
					(min) days_*, ///
					by(plan bene_id) fast

					* Recoding final phase variable to string
					gen finphas = ""
					replace finphas = "XX" if finalphase==1
					replace finphas = "NA" if finalphase==2
					replace finphas = "DD" if finalphase==3
					replace finphas = "DP" if finalphase==4
					replace finphas = "PP" if finalphase==5
					replace finphas = "DI" if finalphase==6
					replace finphas = "PI" if finalphase==7
					replace finphas = "II" if finalphase==8
					replace finphas = "DC" if finalphase==9
					replace finphas = "PC" if finalphase==10
					replace finphas = "IC" if finalphase==11
					replace finphas = "CC" if finalphase==12

					format date_* %td 
					* Save intermediate collapsed claims file
			tempfile claims_`year'_`y'
			save `claims_`year'_`y''
		} // end y
		
		* Append files to create a complete file for 2010
		use `claims_`year'_1', clear
			forv y = 2/10 {
				append using `claims_`year'_`y''
			} // end y
		
		save ${int}/claimsmerge`year', replace
	
	} // end y10
		
} // end foreach year
} // end claim

*****************************************************
********* STEP 3: Clean raw plan data ***************
*****************************************************

if plan == 1  {

forv year = 2007/2010 {

	* Load the plan files for each year
	* if re-running, need to change file paths to reflect where data is 
	if `year'~=2010 {
	use `plan', clear
	}

	if `year'==2010 {
	use `plan_2010', clear
	}

	* Generate indicator for no gap coverage (different coding in 2008 & 2010)
	if `year'~=2008 & `year'~=2010 {
	gen nogapcov1 = (gapcovtp=="N"|gapcovtp=="9")
	}

	if `year' == 2008 {
	gen nogapcov1 = (gapcovtp=="N")
	}

	if `year' == 2010 {
	gen nogapcov1 = (gap_coverage_type=="N"|gap_coverage_type=="9")
	}


	* Generate indicator for no gap
	gen nogap1 = missing(icl_amt)

	* Generate our unique plan ID
	
	if `year' ~= 2010 {
	gen plan1 = plan_id + cntrctid
	}

	if `year' == 2010 {
	gen plan1 = plan_id + contract_id
	
	*standardize names
	rename egwp_indicator egwp_ind
	rename drug_benefit_type drgbentp 
	rename post_oopt_apply pst_oopt
	rename plan_type pln_type
	}


	keep plan1 ded_amt icl_amt egwp_ind nogap1 drgbentp nogapcov1 oopt_amt pst_oopt pln_type
	
	rename nogapcov1 nogapcov
	rename nogap1 nogap
	rename plan1 plan
	

	save ${int}/plnmerge`year', replace

} // end foreach year
} // end plan

**************************************************************
*** STEP 4: Merge based on individual and plan identifiers ***
**************************************************************
if merge == 1 {

forv year = 2007/2010 {

	* Load claims data for the given year
	use ${int}/claimsmerge`year', clear

	* Merge claims data (master) to plan data (using)
	merge m:1 plan using ${int}/plnmerge`year', gen(_merge_pln)

	* Merge claims/plan data (master) to beneficiary data (using)
	merge m:1 bene_id using ${int}/benmerge`year', gen(_merge_ben)

	* We observe 5 merge combinations and drop all but 2:
	* KEEP: 
		* 1. Matched Beneficiaries, Claims, and Plans
		* 2. Matched Beneficiaries and Plans with No Claims (need to recode variables first):
		
			* We need to recode the missing variables to zeroes (for spending and date variables)
			* and the maximum number of days (for censored timing variables)
			* Set locals for the variables we need to recode
			local spendvar count oopspend totcost preicl_* gap_* cat_* ded_oopspend initial_* ///
				ded_totcost total_* m1* m2* m3* m4* m5* m6* m7* m8* m9* w1* w2* w3* w4* w5* jan_* dec_* early_*
			local days days_to_*
			local dates date_*
			local planvar_`year' "ded_amt icl_amt nogap nogapcov oopt_amt pst_oopt pln_type drgbentp"

			preserve
				keep if _merge_ben == 2 // make subset of data with just unmerged
				rename plan plan_temp
				rename plan12 plan //use plan12 (from BEN dataset) to merge with plan cahracteristics
				drop `planvar_`year''
				merge m:1 plan using ${int}/plnmerge`year', keep(match)

				* The observations we keep are beneficiaries with valid plan IDs who do have not filed claims,
				* so we recode them as 0 in spending variables
				foreach var of varlist `spendvar' {
					replace `var'=0
				}
				foreach day of varlist `days' {
					replace `day'=92
				}		
				foreach date of varlist `dates' {
					replace `date'=td(01jan1960)
				}
				replace finphas = "PP" if (ded_amt==0 | missing(ded_amt))
				replace finphas = "DD" if ded_amt>0 & !missing(ded_amt)
				
				gen plan12 = plan
				drop plan_temp _merge
				tempfile beneswithnoclaims
				save `beneswithnoclaims', replace
			restore
			drop if _merge_ben == 2 // after restore, drop all unmatched
			append using `beneswithnoclaims', force // append to replace the drops with merged datapoints

	* DROP: 
		* 3. Claims not matched to beneficiaries
		drop if _merge_ben==1
		* 4. Claims not matched to a plan
		drop if _merge_pln==1
		* 5. Plans not matched to any claims
		drop if _merge_pln==2
		
	* Make first sample selection drops
	* 1. Drop beneficiaries not in the strict 20% Medicare sample
	keep if strctflg=="05"|strctflg=="15"| strctflg=="100"
	* 2. Drop beneficiaries not in the United States
	destring state_cd, gen(statenum)
	drop if statenum == 40 | statenum == 48 | statenum >=54 

	* We now have a dataset at beneficiary-plan level for each year
	save ${int}/bene`year', replace

} // end foreach

} // end merge

********************************************************
**** STEP 5: Append years to make multiyear dataset ****
********************************************************
if append == 1 {

* This section just appends the datasets from each of the three years

* The beneficiary IDs change from 2008-2009 so we convert the 2009 and 2010 ID's to the 2007/2008 ones
use ${int}/bene2009, clear
	merge m:1 bene_id using /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/xwalk20174_16702, gen(_dua_bene)
	drop if _dua_bene==2
	replace bene_id20174=bene_id if bene_id20174==""
	drop bene_id
	rename bene_id20174 bene_id
save ${int}/bene2009_mod, replace

use ${int}/bene2010, clear
	merge m:1 bene_id using /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/xwalk20174_16702, gen(_dua_bene)
	drop if _dua_bene==2
	replace bene_id20174=bene_id if bene_id20174==""
	drop bene_id
	rename bene_id20174 bene_id
save ${int}/bene2010_mod, replace

foreach range in 7080910 70809 {

*append years
use ${int}/bene2007, clear

if `range'==70809 {
	append using ${int}/bene2008 ${int}/bene2009_mod, gen(yr)
}

if `range'==7080910 {
	append using ${int}/bene2008 ${int}/bene2009_mod ${int}/bene2010_mod, gen(yr)
}

	replace yr = yr+2007



* Save a full sample (before additional drops) for summary statistics
	save ${int}/bene0`range'_full_with_dups, replace
	sort bene_id yr plan, stable
	gen rand=runiform()
preserve
	* Drop duplicate beneficiaries
	sort bene_id yr rand, stable
	by bene_id yr: gen dup = _n
	drop if dup!=1
	drop dup rand
	save ${out}/bene0`range'_full, replace
restore


***********************
* SAMPLE RESTRICTIONS *
***********************

* Dropping beneficiairies who join for a reason other than turning 65
drop if age<65
keep if orec=="0"
count

* Dropping beneficiairies who are eligible for Medicaid as well
encode dual_mo, gen(dual)
drop if dual > 1 & dual != . // drop duals
count

* Keeping only beneficiaries with no listed 3rd party cost sharing
keep if no_costsharing==1
keep if crdcovsw=="0"
count

* Keeping only stand-alone Medicare Part D plans
keep if pln_type=="29"
count

* Dropping beneficiaries who switch plans within the year
drop if numswitch!=0 
count

* Dropping beneficiaries who die in the year
drop if death == 1
count

* Dropping any remaining duplicate beneficiaries in the year- (drop randomly)

sort bene_id yr rand, stable
by bene_id yr: gen dup = _n
drop if dup!=1
drop dup rand
count

* Save the full data
save ${int}/bene0`range', replace


} // end range loop
} // end append loop

*****************************************************
**** STEP 6A: Generate baseline analysis dataset ****
*****************************************************
if baseline_restrictions == 1 {

foreach range in 7080910 70809 {

use ${int}/bene0`range', clear

count

* Generating a variable describing gap coverage
gen plan_type=""
replace plan_type = "nogapcov" if nogapcov==1
replace plan_type = "somegapcov" if nogapcov==0
replace plan_type = "no_icl" if icl_amt==.
tab plan_type, mi

* Generating a variable for the standard ICL/Ded amount in each year
gen standardicl = 2400 if yr==2007
replace standardicl = 2510 if yr==2008
replace standardicl=2700 if yr==2009
replace standardicl=2830 if yr==2010
gen has_standardicl= icl_amt==standardicl
gen standardded = 265 if yr==2007
replace standardded = 275 if yr==2008
replace standardded = 295 if yr==2009
replace standardded = 310 if yr==2010
gen has_standardded = ded_amt==standardded

* Generating a variable for the type of plan (deductible/no deductible, gap coverage/no gap coverage)
gen plan_group=""
replace plan_group = "ded_nogap" if ded_amt>0 & ded_amt!=. & nogapcov==1
replace plan_group = "noded_nogap" if ded_amt==0 & nogapcov==1
replace plan_group = "noded_gapcov" if ded_amt==0 & nogapcov==0
replace plan_group = "ded_gapcov" if ded_amt>0 & ded_amt!=. & nogapcov==0
replace plan_group = "no_icl" if icl_amt==.

* Generate cleaner version of final phase variable that eliminates the straddle codes and orders the phases
gen finphas_clean = ""
replace finphas_clean = "A_Ded" if finphas=="DD" // Deductible phase
replace finphas_clean = "B_PreICL" if finphas=="DP"|finphas=="PP" // Pre-ICL phase
replace finphas_clean = "C_ICL" if finphas=="DI"|finphas=="PI"|finphas=="II" // ICL Phase
replace finphas_clean = "D_Cat" if finphas=="DC"|finphas=="PC"|finphas=="IC"|finphas=="CC" // Catastrophic Phase
replace finphas_clean = "NA" if finphas=="NA"

* Generate variables for difference between spending and various kink points
gen icl_diff = totcost-icl_amt
replace icl_diff = totcost-standardicl if icl_amt==.
gen ded_diff = totcost-ded_amt
gen cat_diff = oopspend - oopt_amt

* Generating bins for the data
gen icl_diff_widebin = 50*floor(icl_diff/50)
gen icl_diff_narrowbin = 20*floor(icl_diff/20)

* Generating the number of beneficiaries in each bin
bysort icl_diff_widebin: gen icl_totwide = _N
bysort icl_diff_narrowbin: gen icl_totnarrow = _N

* Generating an indicator for whether beneficiaries stopped claiming before December
foreach cut in any acute chr gen brand maint nonmaint hedis nonhedis {
	gen nocl_01dec_`cut' = month(date_`cut')<12
}

* Empirically calculating beneficiary cost sharing for each phase:
	* Step 1: Calculate the fraction of total cost in a phase paid OOP
	gen bene_coin_ded = ded_oopspend/ded_totcost
	gen bene_coin_preicl = preicl_oopspend/preicl_totcost
	gen bene_coin_gap = gap_oopspend/gap_totcost
	gen bene_coin_cat = cat_oopspend/cat_totcost

	* Eliminating infeasible amounts
	replace bene_coin_ded = 1 if bene_coin_ded>1 & !missing(bene_coin_ded)
	replace bene_coin_preicl = 1 if bene_coin_preicl>1 & !missing(bene_coin_preicl)
	replace bene_coin_gap = 1 if bene_coin_gap>1 & !missing(bene_coin_gap)
	replace bene_coin_cat = 1 if bene_coin_cat>1 & !missing(bene_coin_cat)

	* Step 2: Calculating the average beneficiary cost sharing as the 
	* 	  as the plan-year cost sharing
	bysort plan yr: egen plan_coin_ded = mean(bene_coin_ded)
	by plan yr: egen plan_coin_preicl = mean(bene_coin_preicl)
	by plan yr: egen plan_coin_gap = mean(bene_coin_gap)

	* Step 3: For plan-years and phases (e.g. catastrophic) where there are no or few
	* 	  observation, replace the plan-year average with a plan group-year average
	bysort plan_group yr: egen plan_group_coin_ded = mean(bene_coin_ded)
	by plan_group yr: egen plan_group_coin_preicl = mean(bene_coin_preicl)
	by plan_group yr: egen plan_group_coin_gap = mean(bene_coin_gap)
	by plan_group yr: egen plan_coin_cat = mean(bene_coin_cat)
	replace plan_coin_ded = plan_group_coin_ded if plan_coin_ded==. & plan_group=="ded"
	replace plan_coin_preicl = plan_group_coin_preicl if plan_coin_preicl==.
	replace plan_coin_gap = plan_group_coin_gap if plan_coin_gap==.

 *Drop variable used for the 65 year old analysis but not the full sample
drop initial_total* 
drop days_to_first*

* Generate a separate dataset for the beneficiaries without the standard ICL for placebo/robustness tests
preserve 
	keep if has_standardicl==0
	save ${out}/bene0`range'_placebo.dta, replace
restore

* Merge in variables from externally generated datasets from Medicare Parts A and B
	* Part D risk scores (generated using CMS programs and Part A and B claims)
	merge 1:1 bene_id yr using ${risk_scores}, gen(rxhcc_merge) keep(match master)
	* 2006 Part D risk scores (generated using CMS programs and Part A and B claims)
	merge 1:1 bene_id yr using ${risk_scores_2006}, gen(rx06_merge) keep(match master)
	* Medicare Advantage risk scores (generated using CMS programs and Part A and B claims to predict Parts A and B spending)
	merge 1:1 bene_id yr using ${MA_risk_scores}, gen(mahcc_merge) keep(match master)
	* Month-level utilization of Medicare Parts A and B
	merge 1:1 bene_id yr using ${partsab_util}, gen(_merge_PartsAandB) keep(match master)
		* Recoding variables to zero for beneficiaries not matched
		foreach var of varlist op_admit* er_op_admit* ip_admit* er_ip_admit* {
			replace `var' = 0 if `var'==.
		}
	* Death dates from all available datasets 
	merge m:1 bene_id using ${death_dates}, gen(_merge_death) keep(match master)

* Saving baseline analysis sample including those with non-standard icl
save ${out}/bene0`range'_baseline_allicl, replace

* Drop those wo standard icl for baseline sample
drop if has_standardicl==0

* Saving out baseline analysis sample
save ${out}/bene0`range'_baseline, replace

} // end range loop

} // if `baseline_restrictions' == 1

******************************************************************************
**** STEP 6b: Generate dataset that crosses years 2007-2008 and 2008-2009 ****
**** for Intertemporal Substitution Analysis *********************************
******************************************************************************
if wide == 1 {

use ${out}/bene07080910_baseline_allicl, clear

* Generate intertemporal specific variables
gen more_late_year_spend = (m1_total_any+m2_total_any+m3_total_any+m4_total_any+m5_total_any+m6_total_any)<(m7_total_any+m8_total_any+m9_total_any+m10_total_any+m11_total_any+m12_total_any)
gen excess_spend_v1 = m1_total_any - ((m2_total_any+m3_total_any+m4_total_any+m5_total_any+m6_total_any+m7_total_any+m8_total_any+m9_total_any+m10_total_any+m11_total_any+m12_total_any)/11)
gen excess_spend_v2 = (m1_total_any+m2_total_any) - ((m3_total_any+m4_total_any+m5_total_any+m6_total_any+m7_total_any+m8_total_any+m9_total_any+m10_total_any+m11_total_any+m12_total_any)/5)
gen excess_spend_placebo = m7_total_any - ((m8_total_any+m9_total_any+m10_total_any+m11_total_any+m12_total_any)/5)

* Keep only the variables we want for intertemporal analysis
keep bene_id ded_amt icl_amt icl_diff ded_diff cat_diff nogap nogapcov excess_spend* totcost plan riskscore_CE riskscore_MA ///
icl_diff_narrowbin icl_diff_widebin yr age sex race partd_joinmonth count plan_group more_late_year_spend finphas_clean ///
total* plan_coin* jan* dec* early* ///
m1* m2* m3* m4* m5* m6* m7* m8* m9* ///
w1* w2* w3* w4* w5* ///
fillsize* ///
RXHCC* HCC* er*

xtile rsq_rx = riskscore_CE, n(4)
xtile rsq_ma = riskscore_MA, n(4)

* Save our year 1 datasets
foreach year in 2007 2008 2009 {
preserve
keep if yr == `year'
renvars, postfix(_y1)
rename bene_id_y1 bene_id
save ${int}/age_bene_intertemp_y1_`year', replace
restore 
}

foreach year in 2008 2009 2010 {
preserve
keep if yr == `year'
renvars, postfix(_y2)
rename bene_id_y2 bene_id
save ${int}/age_bene_intertemp_y2_`year', replace
restore 
}

use ${int}/age_bene_intertemp_y1_2009, clear
merge 1:1 bene_id using ${int}/age_bene_intertemp_y2_2010, gen(_merge_0910)
save ${int}/age_bene_intertemp_0910, replace

use ${int}/age_bene_intertemp_y1_2008, clear
merge 1:1 bene_id using ${int}/age_bene_intertemp_y2_2009, gen(_merge_0809)
save ${int}/age_bene_intertemp_0809, replace

use ${int}/age_bene_intertemp_y1_2007, clear
merge 1:1 bene_id using ${int}/age_bene_intertemp_y2_2008, gen(_merge_0708)
save ${int}/age_bene_intertemp_0708, replace

foreach range in "0708_0809" "0708_0809_0910" {

use ${int}/age_bene_intertemp_0708, clear

append using ${int}/age_bene_intertemp_0809

if "`range'"=="0708_0809_0910" {
append using ${int}/age_bene_intertemp_0910
}

/*gen jan_any_y2 = m1_total_any_y2>0 & !missing(m1_total_any_y2)
replace jan_any_y2 = . if missing(m1_total_any_y2)

foreach cut in any acute chr brand gen maint {

	gen frac_jan_`cut'_y2 = 12*(m1_total_`cut'_y2/total_`cut'_y2)
	gen excess_jan_`cut'_y2 = m1_total_`cut'_y2 - (total_`cut'_y2 - m1_total_`cut'_y2)/11

} // foreach cut 
*/

save ${out}/bene_`range'_wide, replace

} // end range loop
} // if wide == 1


log close
