# R code for reporting results of counterfactural simulations
#Rprof(filename="run.Rprof",memory.profiling=TRUE)
library(RColorBrewer)
library(quantreg)
library(ggplot2)
library(data.table)
library(Rcpp)

dir.name <- "cara6.5"
paramfile <- "x.6.5"

#outDir <- "figures/3types/"
#paramfile <- "../estimation/baseline-pc3/smmEstCMA_2014-09-07_23.30.29.txt"
#cfgFile <- "../estimation/baseline-pc3/baseline3.cfg"

if (!exists("outDir") || is.na(outDir)) {
  outDir <- paste("figures/",dir.name,"/",sep="")
}
est.dir <- paste("../estimation/",dir.name,sep="")
if (!exists("paramfile") || is.na(paramfile)) {
  est.dir <- paste("../estimation/",dir.name,sep="")
  pfiles <- paste(est.dir,dir(est.dir,pattern="smmEstCMA(.+).txt"),sep="/")
  paramfile <- pfiles[order(pfiles,decreasing=TRUE)[1]]
  warning(sprintf("Using %s as paramfile",paramfile))
}
if (!exists("cfgFile") || is.na(cfgFile)) {
  est.dir <- paste("../estimation/",dir.name,sep="")
  cfgFile <- paste(est.dir,dir(est.dir,pattern="(.+).cfg$"),sep="/")
  if (length(cfgFile)>1) {
    stop("Cannot determine configuration file")
  }
}
#outDir <- "figures/nogapx/"


## Settings
#datafile <- "../c/data/p20.txt"
datafile <- "../estimation/data/medD-full-const.sl3"
nplanUse <- 500
probKeep <- 0.1
probKeep65 <- 0.1

#paramfile <- "../estimation/6types/smmEstCMA_2014-08-07_22.55.25.txt"
#cfgFile <- "../estimation/6types/6types.cfg"
#outDir <- "figures/6types/"
#cfgFile <- "../estimation/termVexcessJan/termVexcessJan.cfg"
#paramfile <- "../estimation/termVexcessJan/smmEstCMA_2014-08-23_12.47.20.txt"#smmEstCMA_2014-05-31_08.01.08.txt"
#cfgFile <- "../estimation/noCovariates/noCovariatess.cfg"
#paramfile <- "../estimation/noCovariates/smmEstCMA_2014-08-24_20.19.03.txt"
#outDir <- "figures/noCovariates/"
fdims <-1.5*c(5.03937, 3.77953) ## figure dimensions, =128mm x 96mm = dimensions of beamer slide
nsim <- 1 ## simulations per observation
nbin <- 75
colors <- brewer.pal(8,"Dark2") ## colors for figures
colors <- colors[c(1:3,5,8)]
forceFig <- TRUE ## recreate figures even if they already exist?




## read configuration
cfg.txt <- readLines(cfgFile)
cfg.txt <- gsub("true","TRUE",cfg.txt)
cfg.txt <- gsub("false","FALSE",cfg.txt)
cfg <- new.env()
eval(parse(text=cfg.txt),envir=cfg)

NX <- cfg$NX + 1
raEst <- cfg$estRiskAversion
nmix <- cfg$NMIX
nlambda <- cfg$nLambda
singleLambdaRatio <- cfg$singleLambdaRatio
terminalV <- cfg$terminalV
omegaThetaDep <- cfg$omegaThetaDep
if (is.null(omegaThetaDep)) {
  omegaThetaDep <- FALSE
}

## theme to mimic Liran's excel figures
fontFamily <- "Helvetica" ## change to Calibri if it's available
themeLE <- function(base_size=12, base_family=fontFamily) {
  theme_bw() %+replace%
  theme(axis.line = element_line(colour="grey"),
        panel.grid.minor = element_blank(),
        panel.background= element_blank(),
        panel.grid.major.x = element_blank(),
        panel.background = element_rect(fill="white",colour =NA),
        panel.border=element_blank(),
        text = element_text(family=base_family, size=base_size,
          face="plain", hjust=0.5, vjust=0.5, angle=0, lineheight=0.9,
          colour="black"),
        axis.text.x = element_text(angle=60),
        legend.position="bottom")
}
printDev <- function(filename, width=7, height=7, family=fontFamily) {
  cairo_pdf(filename,width=width,height=height,family=family)
  #pdf(filename,width,height)
}


source("param.R")

## I use this file interactively, so time consuming portions of code
## that I do not want to re-run everytime are wrapped in if {} else {}
## blocks
if (!exists("result")) {
  ## run simulations using C++ library
  cflib <- "cf.so"
  if (is.loaded("cfsim")) dyn.unload(cflib)
  dyn.load(cflib)
  rate <- t(matrix(c(1.0, 0.25, 1.0, 0.07,  ## std benefit
                     1.0, 0.25, 0.25, 0.07, ## filled gap
                     1.0, 0.25, 1.0, 0.07,  ## no deductible
                     #1.0, 0.25, 0.07, 0.07, ## generously filled gap
                     0.0, 0.0,  0.0, 0.0,   ## full insurance       
                     1.0, 1.0,  1.0, 1.0,   ## no insurance         
                     1.0, 1.0,  1.0, 0.07   ## only catastrophic    
                     ),nrow=4))
  dic <- t(matrix(c(275, 2510, 4050, ## std benefit
                  275, 2510, 4050, ## filled gap
                  0, 2510, 4050, ## no deductible
                  #275, 2510, 4050, ## generously filled gap
                  275, 2510, 4050, ## full insurance       
                  275, 2510, 4050, ## no insurance         
                  275, 2510, 4050  ## only catastrophic    
                  ), nrow=3))
  cfname <- c("standard benefit",
              "filled gap",
              "no deductible",
              ##"more filled gap",
              "full insurance",
              "no insurance",
              "only catastrophic coverage")
  tmp.file <- paste("x",as.numeric(Sys.time()),".tmp",sep="")
  pm <- parameters(paramfile,tmp.file)
  args <- list(datafile,tmp.file,nsim,rate,dic,compStatics=FALSE,probKeep,probKeep65,nplanUse,cfgFile)
  names(args) <- c("datafile","xfile","nsim","rate","dic","compStatics","probKeep","probKeep65","nplanUse",
                   "cfgFile")
  result <- .Call("cfsim",args)
  unlink(tmp.file)
} else {
  warning("Simulation result already exists, not re-running.")
}

if (!exists("simCost")) {
  ## put results into nice data structures
  id <- 1:length(result$Observed$plan)
  ## constant data
  conData <- data.frame(id,result$Observed[c("joinweek","plan","ded","icl","cat",
                                             "weight","age","rs","nogap")])
  conStd <- data.frame(id,result$cf[[1]][c("joinweek","plan","ded","icl","cat",
                                           "weight","age","rs","nogap")])
  nhData <- data.frame(id,result$NoHole[c("joinweek","plan","ded","icl","cat",
                                          "weight","age","rs","nogap")])
  ndData <- data.frame(id,result$NoDed[c("joinweek","plan","ded","icl","cat",
                                         "weight","age","rs","nogap")])  
  ## observed cost data
  N <- length(id)
  T <- length(result$Observed$cost)/N
  idc <- as.vector(rep(1,T) %*% t(id))
  tc <- as.vector((1:T) %*% t(rep(1,N))) - 1
  jw <- as.vector(rep(1,T) %*% t(conData$joinweek))
  costData <- data.frame(idc,tc,result$Observed[c("cost","cumCost")],jw)
  names(costData)[1:2] <- c("id","t")
  costData[costData$t<costData$jw,c("cost","cumCost")] <- 0
  costData <- costData[with(costData,order(id,t)),]
  ## simulated cost data
  nsim <- length(result$Simulated)
  simDF <- function(resultList,cdat) {
    jw <- as.vector(rep(1,T) %*% t(cdat$joinweek))
    df <- data.frame(idc,tc,resultList[[1]][c("cost","cumCost")],jw,resultList[[1]]$delay)
    df$sim <- 1
    if (nsim>1) {
      for(s in 2:nsim) {
        sim <- rep(s,length(idc))
        df <- rbind(df,
                    data.frame(idc,tc,resultList[[s]][c("cost","cumCost")],jw,
                               resultList[[s]]$delay,sim))
      }
    }
    names(df) <- c(names(costData),"delay","sim")
    df[df$t<df$jw,c("cost","cumCost","delay")] <- 0
    df <- df[with(df,order(sim,id,t)),]
    return(df)   
  }
  scf <- list()
  simCost <- simDF(result$Simulated,conData)
  sim0 <- simDF(result$SimJoinZero,conData)
  simNoHole <- simDF(result$SimNoHole,nhData)
  simNoDed <- simDF(result$SimNoDed,ndData)
  stdB <- simDF(result$scf[[1]],conStd)
  stdBnh <- simDF(result$scf[[2]],conStd)
  dEqSnh <-simDF(result$scf[[3]],conStd)
  ##scf[[4]] <- simDF(result$scf[[4]],conStd)
  fullIns <- simDF(result$scf[[4]],conStd)
  noIns <- simDF(result$scf[[5]],conStd)
  scf[[6]] <- simDF(result$scf[[6]],conStd)
} else {
  warning("Simulation results already in data.frames, not re-creating.")
}
T <- max(costData$t)

## create output directory if needed
if (!file.exists(outDir)) {
  dir.create(outDir,recursive=TRUE)
}


## weighted quantile, sd, and median
weighted.quantile <- function(x,probs=seq(0,1,0.25),w) {
  if (length(probs)>1) rq(x ~ 1, tau=probs,weights=w)$fitted.values[1,]
  else rq(x ~ 1, tau=probs,weights=w)$fitted.values[1]
}
weighted.median <- function(x,w) {
  weighted.quantile(x,0.5,w)
}
weighted.sd <- function(x,w) {
  xm <- weighted.mean(x,w)
  sqrt(sum(w*(x-xm)^2)/sum(w))
}

bnames <- c("0","RS","65","nogap")
## reorder types by total spending
spend.full.ins <- 52* (pm$lam %*% pm$marginalProb) *exp(pm$mu + pm$sig^2/2)
ord <- order(spend.full.ins)
nd <- 3
filename <- paste(outDir,"param.tex",sep="")
#sink()
cat("$\\delta$ & ",format(pm$delta,digits=nd), file=filename,append=FALSE)
if (raEst) {
  cat("& risk aversion & ",format(pm$riskAversion,digits=nd), file=filename,append=TRUE)
}
cat(" \\\\ \n", file=filename,append=TRUE)
if (terminalV) {
  cat("& $\\delta_\\omega$ & ",format(pm$deltaOmega,digits=nd), file=filename,append=TRUE)
  cat("& $\\delta_\\theta$ & ",format(pm$deltaTheta,,digits=nd), file=filename,append=TRUE)
  cat(" \\\\ \n", file=filename,append=TRUE)  
}
cat(" ",sprintf("$j=%i$",1:nmix),sep=" & ",file=filename,append=TRUE); cat(" \\\\ \n",file=filename,append=TRUE)
for(j in 1:ncol(pm$bmix)) {
  cat(sprintf("$\\beta_{%s}$",bnames[j]),format(pm$bmix[ord,j],digits=nd),sep=" & ",file=filename,append=TRUE);
  cat(" \\\\ \n",file=filename,append=TRUE)
}
cat("$\\mu$",format(pm$mu[ord],digits=nd),sep=" & ",file=filename,append=TRUE);
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$\\sigma$",format(pm$sig[ord],digits=nd),sep=" & ",file=filename,append=TRUE);
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$p_\\omega$",format(pm$pom[ord],digits=nd),sep=" & ",file=filename,append=TRUE);
cat(" \\\\ \n",file=filename,append=TRUE)
for (i in 1:nlambda) {
  cat("$\\lambda$",format(pm$lam[ord,i],digits=nd),sep=" & ",file=filename,append=TRUE);
  cat(" \\\\ \n",file=filename,append=TRUE)
}
cat("\\\\ \n",file=filename,append=TRUE)
cat("\\multicolumn{6}{l}{$\\lambda$ transition probabilities} \\\\ \n",file=filename,append=TRUE);
for(i in 1:nlambda) {
  cat(" & " , format(pm$transitionProb[i,],digist=nd), sep=" & ", file=filename,append=TRUE);
  cat(" \\\\ \n",file=filename,append=TRUE)
}
cat("\\multicolumn{6}{l}{$\\lambda$ marginal probabilities} \\\\ \n",file=filename,append=TRUE);
for(i in 1:nlambda) {
  cat(" & " , format(pm$marginalProb[i],digist=nd), sep=" & ", file=filename,append=TRUE);
  cat(" \\\\ \n",file=filename,append=TRUE)
}
cat("\\\\ \n",file=filename,append=TRUE)
if (omegaThetaDep) {
  cat("$\\rho$ & ", format(pm$pOT, digits=nd), "\\\\ \n", file=filename,append=TRUE);
}

filename <- paste(outDir,"param2.tex",sep="")
if (NX==1) {
  xbj <- cbind(rep(1,nrow(conData))) %*% t(pm$bmix)
} else if (NX==3) {
  xbj <- cbind(rep(1,nrow(conData)), conData$rs, (conData$age==65)) %*% t(pm$bmix)
} else if (NX==4) {
  xbj <- cbind(rep(1,nrow(conData)), conData$rs, (conData$age==65), conData$nogap) %*% t(pm$bmix)
}
pj <- exp(xbj)/rowSums(exp(xbj))
if (NX>1) {
  dpdrs <- matrix(rep(pm$bmix[,2],nrow(pj)),nrow=nrow(pj), byrow=TRUE) * pj -
    pj*rowSums(matrix(rep(pm$bmix[,2],nrow(pj)),nrow=nrow(pj), byrow=TRUE) * pj)  
} else {
  dpdrs <- matrix(0,nrow=nrow(conData),ncol=nmix)
}
cat("$\\mathrm{P}(j)$", sprintf("%.2f",colMeans(pj[,ord])),sep=" & ",file=filename,append=FALSE)
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$\\mathrm{P}(j|\\text{age}=65)$", sprintf("%.2f",colMeans(pj[conData$age==65,ord])),sep=" & ",file=filename,append=TRUE)
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$\\mathrm{P}(j|\\text{age}>65)$", sprintf("%.2f",colMeans(pj[conData$age>65,ord])),sep=" & ",file=filename,append=TRUE)
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$\\mathrm{E}\\left[ \\frac{d \\mathrm{P}(j|\\text{rs})}{d \\text{rs}} \\right]$",
    sprintf("%.2f",colMeans(dpdrs[,ord])),sep=" & ",file=filename,append=TRUE)
cat(" \\\\ \n",file=filename,append=TRUE)

cat("$\\mathrm{E}[\\theta|j]$", sprintf("%.2f",exp(pm$mu + pm$sig^2/2)[ord]),sep=" & ",file=filename,append=TRUE)
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$\\mathrm{s.d.}(\\theta|j)$", sprintf("%.0f",(sqrt((exp(pm$sig^2)-1)*exp(2*pm$mu + pm$sig^2)))[ord]),
    sep=" & ",file=filename,append=TRUE)
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$\\mathrm{E}[\\text{spend full ins.}|j]$",
    sprintf("%.2f",(52*pm$lam*exp(pm$mu + pm$sig^2/2))[ord]),sep=" & ",file=filename,append=TRUE)
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$\\mathrm{E}[\\text{spend 0.25 coins.}|j]$",
    sprintf("%.2f",(52*pm$lam*((1-pm$pom) + pm$pom*0.75)*exp(pm$mu + pm$sig^2/2))[ord])
            ,sep=" & ",file=filename,append=TRUE)
cat(" \\\\ \n",file=filename,append=TRUE)
cat("$\\mathrm{E}[\\text{spend no ins.}|j]$",
    sprintf("%.2f",(52*pm$lam*((1-pm$pom) + pm$pom*0.0)*exp(pm$mu + pm$sig^2/2))[ord])
    ,sep=" & ",file=filename,append=TRUE)
cat(" \\\\ \n",file=filename,append=TRUE)
if (raEst) {
  q <- weighted.quantile(costData$cumCost[costData$t==T],c(0.05,0.95),conData$weight)
  ceq <- log(sum(exp(pm$riskAversion*q)*c(0.5,0.5)))/pm$riskAversion
  cat(sprintf("\\multicolumn{6}{l}{{\\tiny{Certainty equivalent of 50/50 gamble between %.0f (5th \\%%tile) and %.0f (95th \\%%tile) annual spending = %.0f}}} \\\\ \n",
              q[1],q[2],ceq),              
      file=filename,append=TRUE)
}


figData <- list()


## histogram of total spending
filename <- paste(outDir,"fit.pdf",sep="")
if (!file.exists(filename) || forceFig) {
  maxc <- 6000
  fd <- rbind(data.frame(x=subset(costData,t==T)$cumCost,w=conData$weight,group="observed"),
              data.frame(x=subset(simCost,t==T)$cumCost,w=conData$weight,group="estimated"))
  tab <- data.table(fd)[,list(p0=weighted.mean(x==0,w),
                              mean=weighted.mean(x,w),
                              sd=weighted.sd(x,w),
                              median=weighted.median(x,w)),by=group]
  fd$x[fd$x>maxc] <- maxc
  fitPlot <- ggplot(data=fd, aes(x=x,y=2*..count../sum(..count..),fill=group,weight=w)) + themeLE() + 
    stat_bin(breaks=seq(0.0,maxc+1,by=100),alpha=0.3,position="dodge") + 
      #geom_histogram(alpha=0.3,position="dodge",binwidth=maxc/nbin) +    
    scale_x_continuous(name="total spending") +
    scale_y_continuous(name="portion") + 
    scale_fill_manual(values=colors) + 
    theme(legend.position = "bottom",legend.title=element_blank()) +
    annotate("text",x=500,y=.06,
             label=sprintf(" observed: P(0)=%.3f median=%.2f, mean=%.2f, and sd=%.2f",
               tab$p0[1],tab$median[1],tab$mean[1],tab$sd[1]), size=4, hjust=0,color=colors[1]) + 
    annotate("text",x=500,y=.05,
             label=sprintf("estimated: P(0)=%.3f median=%.2f, mean=%.2f, and sd=%.2f",
               tab$p0[2],tab$median[2],tab$mean[2],tab$sd[2]), size=4, hjust=0,color=colors[2]) 
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  figData <- c(figData, list(ggplot_build(fitPlot)$data[[1]]))
  names(figData)[length(figData)] <- "Fit.Histogram"
} else {
  warning(paste(filename,"exists, not recreating"))
}
    
## total spending near icl
filename <- paste(outDir,"bunchFit.pdf",sep="")
if (!file.exists(filename) || forceFig) {  
  fd <- rbind(data.frame(x=(subset(costData,t==T)$cumCost-conData$icl),w=conData$weight,
                         group="observed"),
              data.frame(x=(subset(simCost,t==T)$cumCost-conData$icl),w=conData$weight,
                         group="estimated"))
  xlim <- c(-1000,1000)
  xld <- xlim
  fd <- subset(fd,x>=xlim[1] & x<=xlim[2])
  bw <- 20
  fitPlot <- ggplot(data=fd, aes(x=x,y=..density..,fill=group, weight=w)) + themeLE() + 
    geom_histogram(alpha=0.3,position="dodge",binwidth=bw) +
    scale_x_continuous(name="total spending - ICL") +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())  
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  figData <- c(figData, ggplot_build(fitPlot)$data[1:2])  
  names(figData)[(length(figData)-1):length(figData)] <-
    c("Fit.Histogram.NearICL","Fit.Density.NearICL")

  filename <- paste(outDir,"bunchFit-estWindow.pdf",sep="")
  fd <- rbind(data.frame(x=(subset(costData,t==T)$cumCost-conData$icl),
                         group="observed",w=conData$weight),
              data.frame(x=(subset(simCost,t==T)$cumCost-conData$icl),
                         group="estimated",w=conData$weight))
  xlim <- c(-500,500)
  xld <- xlim
  fd <- subset(fd,x>=xlim[1] & x<=xlim[2])
  bw <- (xlim[2]-xlim[1])/nbin
  fitPlot <- ggplot(data=fd, aes(x=x,y=..density..,fill=group,weight=w)) + themeLE() + 
    geom_histogram(alpha=0.3,position="dodge",binwidth=bw) +
    scale_x_continuous(name="total spending - ICL") +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())  
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  figData <- c(figData, ggplot_build(fitPlot)$data[1:2])  
  names(figData)[(length(figData)-1):length(figData)] <-
    c("Fit.Histogram.NearICL2","Fit.Density.NearICL2")
} else {
  warning(paste(filename,"exists, not recreating"))
}

## Timing moments
month.names <- c("January","February","March","April",
                 "May","June","July","August","September",
                 "October","November","December")
filename <- paste(outDir,"timing",month.names[12],".pdf",sep="")
if (!file.exists(filename) || forceFig) { 
  fd <- rbind(data.frame(x=(subset(costData,t==T)$cumCost-conData$icl),
                         group="observed",w=conData$weight),
              data.frame(x=(subset(simCost,t==T)$cumCost-conData$icl),
                         group="estimated",w=conData$weight))
  xlim <- c(-500,500)
  bins <- seq(-500,500,length.out=21)
  fd$bin <- NA
  for(i in 1:(length(bins)-1)) {
    fd$bin[fd$x>=bins[i] & fd$x<bins[i+1]] <- i
  }
  for (m in 6:12) {
    t <- T-(13-m)*4
    fd$y <- c((rowsum(costData$cost*(costData$t>=t),group=costData$id)>0),
              (rowsum(simCost$cost*(simCost$t>=t),group=costData$id)>0))
    y.obs <- rep(NA,length(bins)-1)
    y.sim <- y.obs
    for(i in 1:(length(bins)-1)) {
      y.obs[i] <- sum(fd$y[fd$group=="observed" & fd$bin==i],na.rm=TRUE) /
        sum(fd$bin==i & fd$group=="observed",na.rm=TRUE)
      y.sim[i] <- sum(fd$y[fd$group=="estimated" & fd$bin==i],na.rm=TRUE) /
        sum(fd$bin==i & fd$group=="estimated",na.rm=TRUE)
    }
    gd <- data.frame(x=(bins[2:length(bins)]+bins[1:(length(bins))-1])/2,
                     observed=y.obs,estimated=y.sim)
    filename <- paste(outDir,"timing",month.names[m],".pdf",sep="")
    fitPlot <- ggplot(data=gd, aes(x=x, y=observed,
                        color="observed", linetype="observed")) +
      themeLE() + 
      geom_line() + 
      geom_line(aes(y=estimated,color="estimated",linetype="estimated")) + 
      scale_x_continuous(name="total spending - ICL") +
      scale_y_continuous(name=paste("P(claim in ",
                         month.names[m],
                         " or later | total spending)",sep="")) + 
      scale_fill_manual(values=colors) +
      scale_colour_manual(values=colors) +
      theme(legend.position = "bottom", legend.title=element_blank())  
    printDev(filename,width=fdims[1],height=fdims[2])
    print(fitPlot)
    dev.off()
  }
  p.obs <- matrix(NA,nrow=13,ncol=2)
  p.sim <- matrix(NA,nrow=13,ncol=2)
  fd$bin <- NA
  fd$bin[fd$x>=-800 & fd$x<=-500] <- 1
  fd$bin[fd$x>=-150 & fd$x<=150] <- 2
  fd$bin[is.na(fd$bin)] <- 0
  for (m in 1:13) {
    m.ind <- costData$t==(m-1)*4 | costData$t==(m-1)*4+1 |
      costData$t==(m-1)*4+2 | costData$t==(m-1)*4+3
    y <- rowsum(costData$cost*m.ind,group=costData$id)>0
    for(i in 1:2) {
      p.obs[m,i] <- mean(y[fd$bin[fd$group=="observed"]==i])
    }
    m.ind <- simCost$t==(m-1)*4 | simCost$t==(m-1)*4+1 |
      simCost$t==(m-1)*4+2 | simCost$t==(m-1)*4+3
    y <- rowsum(simCost$cost*m.ind,group=simCost$id)>0
    for(i in 1:2) {
      p.sim[m,i] <- mean(y[fd$bin[fd$group=="estimated"]==i])
    }
  }
  gd <- data.frame(month=1:13,obs.pre=p.obs[,1],obs.kink=p.obs[,2],
                   sim.pre=p.sim[,1], sim.kink=p.sim[,2])
  fitPlot <- ggplot(data=gd, aes(x=month, y=obs.pre,
                    color="pre")) + 
    themeLE() + 
    geom_line(linetype=1) + 
    geom_line(aes(y=obs.kink,color="kink"),linetype=1) +
    geom_line(aes(y=sim.pre,color="pre"),linetype=2) +
    geom_line(aes(y=sim.kink,color="kink"),linetype=2) +
    scale_x_continuous(name="month") +
    scale_y_continuous(name="P(claim in month|total spending)") +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())
  filename <- paste(outDir,"timingAll.pdf",sep="")
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()

  figData <- c(figData, list(gd))
  names(figData)[length(figData)] <- "timingAll"  
} else {
  warning(paste(filename,"exists, not recreating"))
}

## Autocorrelation of claims
filename <- paste(outDir,"autocorr.pdf",sep="")
if (!file.exists(filename) || forceFig) {
  lags <- 20
  lag <- 0:lags
  cost.obs <- matrix(NA,ncol=(T+1),nrow=N)
  cost.sim <- cost.obs
  cost.obs[costData$id + costData$t*N] <- costData$cost
  cost.sim[simCost$id + simCost$t*N] <- simCost$cost
  p.obs <- rep(NA,lags+1)
  p.sim <- p.obs
  for(l in lag) {
    p.obs[l+1] <- mean(cost.obs[,1:(T+1-l)]>0 & cost.obs[,(1+l):(T+1)]>0,na.rm=TRUE)
    p.sim[l+1] <- mean(cost.sim[,1:(T+1-l)]>0 & cost.sim[,(1+l):(T+1)]>0,na.rm=TRUE)
  }
  gd <- data.frame(lag=lag,observed=p.obs,estimated=p.sim)
  fitPlot <- ggplot(data=gd, aes(x=lag,y=observed,color="observed",linetype="observed")) + themeLE() +
    geom_line() + 
    geom_line(aes(y=estimated,color="estimated",linetype="estimated")) + 
    scale_x_continuous(name="lag") +
    scale_y_continuous(name=paste("P(claim t & t-lag)",sep="")) +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())  
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()

  lags <- 20
  lag <- 1:lags
  cost.obs <- matrix(NA,ncol=(T+1),nrow=N)
  cost.sim <- cost.obs
  cost.obs[costData$id + costData$t*N] <- costData$cost
  cost.sim[simCost$id + simCost$t*N] <- simCost$cost
  p.obs <- rep(NA,lags)
  p.sim <- p.obs
  fe.obs <- p.obs
  fe.sim <- p.sim
  fe.theta <- fe.sim
  cor.li <- p.obs
  
  theta <- matrix(fullIns$cost,nrow=(T+1),ncol=N)
  li <- t(matrix(result$scf[[1]][[1]]$lambdaIndex,nrow=(T+1),ncol=N))

  e.obs <- cost.obs - rowMeans(cost.obs,na.rm=TRUE)
  e.sim <- cost.sim - rowMeans(cost.sim,na.rm=TRUE)
  e.theta <- t(theta) - rowMeans(t(theta),na.rm=TRUE)

  for(l in lag) {
    c1 <- as.vector
    p.obs[l] <- cor(as.vector(cost.obs[,1:(T+1-l)]),as.vector(cost.obs[,(1+l):(T+1)]),
                    use="na.or.complete")
    p.sim[l] <-  cor(as.vector(cost.sim[,1:(T+1-l)]),as.vector(cost.sim[,(1+l):(T+1)]),
                     use="na.or.complete")

    fe.obs[l] <- cor(as.vector(e.obs[,1:(T+1-l)]),as.vector(e.obs[,(1+l):(T+1)]),
                    use="na.or.complete")
    fe.sim[l] <-  cor(as.vector(e.sim[,1:(T+1-l)]),as.vector(e.sim[,(1+l):(T+1)]),
                    use="na.or.complete")
    fe.theta[l] <-  cor(as.vector(e.theta[,1:(T+1-l)]),as.vector(e.theta[,(1+l):(T+1)]),
                    use="na.or.complete")
    cor.li[l] <-   cor(as.vector(li[,1:(T+1-l)]),as.vector(li[,(1+l):(T+1)]),
                    use="na.or.complete")
    
  }
  gd <- data.frame(lag=lag,observed=p.obs,estimated=p.sim)
  fitPlot <- ggplot(data=gd, aes(x=lag,y=observed,color="observed",linetype="observed")) + themeLE() +
    geom_line() + 
    geom_line(aes(y=estimated,color="estimated",linetype="estimated")) + 
    scale_x_continuous(name="lag") +
    scale_y_continuous(name=paste("Correlation spending at time t and t-lag",sep="")) +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())
  filename <- paste(outDir,"autocorr-spend.pdf",sep="")
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()

  gd <- data.frame(lag=lag,observed=fe.obs,estimated=fe.sim)
  fitPlot <- ggplot(data=gd, aes(x=lag,y=observed,color="observed",linetype="observed")) + themeLE() +
    geom_line() + 
    geom_line(aes(y=estimated,color="estimated",linetype="estimated")) + 
    scale_x_continuous(name="lag") +
    scale_y_continuous(name=paste("Correlation spending at time t and t-lag",sep="")) +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())
  filename <- paste(outDir,"autocorr-spend-fe.pdf",sep="")
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()

} else {
  warning(paste(filename,"exists, not recreating"))
}
  

## Initial spending by join quarter and deductible
filename <- paste(outDir,"initial.tex",sep="")
if (!file.exists(filename) || forceFig) {
  library(xtable)
  injw <- conData$joinweek+11<=max(costData$t) 
  weeks <- ave(costData$cumCost==0, costData$id, FUN=sum)[costData$t==costData$jw+11]
  maxw <- 13
  weeks[weeks>maxw] <- maxw
  ispend <- costData$cumCost[costData$t==costData$jw+11]
  iclaim <- ispend>0
  quarter <- floor((conData$joinweek[injw]-4)/((T+1)/4))
  ded <- conData$ded[injw]>0
  fd <- data.frame(weeks,ispend,iclaim,quarter,ded,group="observed")
  fd <- subset(fd,conData$age[injw]==65)
  obs <- data.table(fd)[,list(weeks=mean(weeks),spend=mean(ispend),pclaim=mean(iclaim)),
                        by=c("quarter","ded")]
  weeks <- ave(simCost$cumCost==0, simCost$id, simCost$sim, FUN=sum)[simCost$t==simCost$jw+11] 
  weeks[weeks>maxw] <- maxw
  ins <- which(simCost$t==simCost$jw+11)
  sid <- simCost$id[ins]
  ispend <- simCost$cumCost[ins]
  iclaim <- ispend>0
  q <- floor((conData$joinweek[sid]-4)/((T+1)/4))
  ded <- conData$ded[sid]>0
  fde <- data.frame(weeks,ispend,iclaim,quarter=q,ded,group="estimated")
  fde <- subset(fde,conData$age[sid]==65)
  fd <- rbind(fd,fde)
  tab <- data.table(fd)[,list(weeks=mean(weeks),spend=mean(ispend),pclaim=mean(iclaim)),
                      by=c("group","quarter","ded")]
  tab <- tab[order(tab$ded,tab$quarter,tab$group),]
  df <- data.frame(tab)
  df$ded <- as.factor(df$ded)  
  cat("\\begin{table}[ht]\\begin{center}\\begin{tabular}{ccccc}
 & \\multicolumn{2}{c}{No deductible} & \\multicolumn{2}{c}{Deductible} \\\\
 Quarter & Observed & Estimated & Observed & Esitmated \\\\ \n",file=filename,append=FALSE)
  for(j in 0:2) {
    cat(sprintf(" %d & %.2f & %.2f & %.2f & %.2f \\\\ \n",
                j, df$weeks[df$quarter==j & df$group=="observed" & df$ded==FALSE],
                df$weeks[df$quarter==j & df$group=="estimated" & df$ded==FALSE],
                df$weeks[df$quarter==j & df$group=="observed" & df$ded==TRUE],
                df$weeks[df$quarter==j & df$group=="estimated" & df$ded==TRUE]),
        file=filename,append=TRUE)
  }
  cat("\\end{tabular}\\end{center}\\end{table}\n",file=filename,append=TRUE)

  ## cleanup
  rm(list=c("q","ded","fde","ins","sid","ispend","iclaim",
       "weeks","injw","quarter","fd","tab","df"))
} else {
  warning(paste(filename,"exists, not recreating"))
}


######################################################################
filename <- paste(outDir,"nogapHist.pdf",sep="")
if (!file.exists(filename) || forceFig) {
  maxc <- 6000
  fd <- rbind(data.frame(x=subset(sim0,t==T)$cumCost,group="baseline",w=conData$weight),
              data.frame(x=subset(simNoHole,t==T)$cumCost,group="filled gap",w=conData$weight))
  tab <- data.table(fd)[,list(mean=weighted.mean(x,w),
                              sd=weighted.sd(x,w),
                              median=weighted.median(x,w)),by=group]
  fd$x[fd$x>maxc] <- maxc
  fitPlot <- ggplot(data=fd, aes(x=x,y=..density..,fill=group,weight=w)) + themeLE() + 
    geom_histogram(alpha=0.3,position="dodge",binwidth=maxc/nbin) +
    scale_x_continuous(name="total spending") +
    scale_fill_manual(values=colors[2:3]) +
    theme(legend.position = "bottom", legend.title=element_blank()) +
    annotate("text",x=500,y=.00275,
             label=sprintf("   baseline: median=%.2f, mean=%.2f, and sd=%.2f",
             tab$median[1],tab$mean[1],tab$sd[1]), size=4, hjust=0,color=colors[2])  +
    annotate("text",x=500,y=.0026,
             label=sprintf(" filled gap: median=%.2f, mean=%.2f, and sd=%.2f",
             tab$median[2],tab$mean[2],tab$sd[2]), size=4, hjust=0,color=colors[3]) 
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  figData <- c(figData,list(ggplot_build(fitPlot)$data[[1]]))
  names(figData)[length(figData)] <- "CF.Hist.ObservedPlans"  

} else {
  warning(paste(filename,"exists, not recreating"))
}

## histogram of spending change without hole
filename <- paste(outDir,"nogapChange.pdf",sep="")
if (!file.exists(filename) || forceFig) {
  maxc <- 5000
  minc <- -2000
  fd <-data.frame(x=(subset(simNoHole,t==T)$cumCost) - subset(sim0,t==T)$cumCost,
                  w=conData$weight)
  fd$x[fd$x>maxc] <- maxc
  fd$x[fd$x<minc] <- minc
  p0 <- weighted.mean(fd$x==0,fd$w)
  fd <- subset(fd, x!=0)
  fitPlot <- ggplot(data=fd, aes(x=x,y=..density..,fill="this does nothing",weight=w)) + themeLE() + 
    geom_histogram(alpha=0.7,position="dodge",binwidth=maxc/nbin) +
    scale_x_continuous(name="change in spending | change ≠ 0") +
    scale_fill_manual(values=colors[3],guide="none") +
    annotate("text",x=0,y=.0045,label=sprintf("P(change=0) = %.3f",p0), size=4, hjust=0) 
  printDev(paste(outDir,"nogapChange.pdf",sep=""),width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  figData <- c(figData, list(ggplot_build(fitPlot)$data[[1]]))
  names(figData)[length(figData)] <- "FilledGapChange.ObservedPlans"
} else {
  warning(paste(filename,"exists, not recreating"))
}

################################################################################
## Counterfactuals using standard benefit
cat(sprintf("%.2f",result$deq),file=paste(outDir,"deq.tex",sep=""),append=FALSE)
filename <- paste(outDir,"std_nogapHist.pdf",sep="")
if (!file.exists(filename) || forceFig) {
  ## histogram of spending with & without hole
  maxc <- 6000
  fd <- rbind(data.frame(x=subset(stdB,t==T)$cumCost,group="baseline",w=conData$weight),
              data.frame(x=subset(stdBnh,t==T)$cumCost,group="filled gap",w=conData$weight),
              data.frame(x=subset(dEqSnh,t==T)$cumCost,group="reduced deductible",w=conData$weight))
  tab <- data.table(fd)[,list(mean=weighted.mean(x,w),
                              sd=weighted.sd(x,w),
                              median=weighted.median(x,w)),by=group]
  fd$x[fd$x>maxc] <- maxc
  fitPlot <- ggplot(data=fd, aes(x=x,y=..density..,fill=group,weight=w)) + themeLE() + 
    geom_histogram(alpha=0.7,position="dodge",binwidth=maxc/nbin) +
    scale_x_continuous(name="total spending") +
    scale_fill_manual(values=colors[2:4]) +
    theme(legend.position = "bottom", legend.title=element_blank()) +
    annotate("text",x=500,y=.00275,
             label=sprintf("     baseline: median=%.2f, mean=%.2f, and sd=%.2f",
               tab$median[1],tab$mean[1],tab$sd[1]), size=4, hjust=0,color=colors[2])  +
    annotate("text",x=500,y=.0025,
             label=sprintf(" filled gap: median=%.2f, mean=%.2f, and sd=%.2f",
               tab$median[2],tab$mean[2],tab$sd[2]), size=4, hjust=0,color=colors[3])  +
    annotate("text",x=500,y=.00225,
             label=sprintf("reduced deductible: median=%.2f, mean=%.2f, and sd=%.2f",
               tab$median[3],tab$mean[3],tab$sd[3]), size=4, hjust=0,color=colors[4])

  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  figData <- c(figData, list(ggplot_build(fitPlot)$data[[1]]))
  names(figData)[length(figData)] <- "CF.Hist.StdBenefit"
  
  ## summary statistics of spending with and without hole
  filename <- paste(outDir,"stdSpending.tex",sep="")

  totalSpend <- function(df) {
    T <-  max(df$t) - min(df$t)+1
    N <- nrow(df)/T
    return(as.vector(subset(df,t==(T-1))$cumCost +
                     pm$deltaTheta^(T:1) %*% matrix(df$delay,nrow=T,ncol=N)))
  }

  fd <- rbind(data.frame(x=totalSpend(stdB),group="baseline",w=conData$weight),
              data.frame(x=totalSpend(stdBnh),group="filled gap",w=conData$weight),
              data.frame(x=totalSpend(dEqSnh),group="zero deductible",w=conData$weight),
              data.frame(x=totalSpend(noIns),group="no insurance",w=conData$weight),
              data.frame(x=totalSpend(fullIns),group="full insurance",w=conData$weight),
              data.frame(x=totalSpend(sim0),group="baseline - actual plans",w=conData$weight),
              data.frame(x=totalSpend(simNoDed),group="zero deductible - actual plans",w=conData$weight), 
              data.frame(x=totalSpend(simNoHole),group="filled gap - actual plans",w=conData$weight)
              )
  tab <- data.table(fd)[,list(mean=weighted.mean(x,w),sd=weighted.sd(x,w),
                              Q25=weighted.quantile(x,probs=.25,w),
                              Q50=weighted.quantile(x,probs=.50,w),
                              Q75=weighted.quantile(x,probs=.75,w),
                              Q90=weighted.quantile(x,probs=.90,w)
                              ),by=group]
  print(xtable(tab,align=rep("c",8),digits=0,display=c("s","s",rep("f",6))),
        file=filename,type="latex")  
  
  ## summary statistics of oop spending with and without hole
  filename <- paste(outDir,"stdOop.tex",sep="")
  totoop <- function(resultList) {
    oop <- resultList[[1]]$totOop
    if (nsim>1) {
      for(s in 2:nsim) {
        oop <- c(oop,resultList[[s]]$totOop)
      }
    }
    return(oop)
  }
  fdo <- rbind(data.frame(x=totoop(result$scf[[1]]),group="baseline",w=conData$weight),
               data.frame(x=totoop(result$scf[[2]]),group="filled gap",w=conData$weight),
               data.frame(x=totoop(result$scf[[3]]),group="zero deductible",w=conData$weight),
                data.frame(x=totoop(result$scf[[5]]),group="no insurance",w=conData$weight),
               data.frame(x=totoop(result$scf[[4]]),group="full insurance",w=conData$weight),
               data.frame(x=totoop(result$SimJoinZero),group="baseline - actual plans",w=conData$weight),
               data.frame(x=totoop(result$SimNoDed),group="zero deductible - actual plans",w=conData$weight),
               data.frame(x=totoop(result$SimNoHole),group="filled gap - actual plans",w=conData$weight)
              )
  tabo <- data.table(fdo)[,list(mean=weighted.mean(x,w),
                                sd=weighted.sd(x,w),
                              Q25=weighted.quantile(x,probs=.25,w),
                              Q50=weighted.quantile(x,probs=.50,w),
                              Q75=weighted.quantile(x,probs=.75,w),
                              Q90=weighted.quantile(x,probs=.90,w)
                              ),by=group]
  print(xtable(tabo,align=rep("c",8),digits=0,display=c("s","s",rep("f",6))),
        file=filename,type="latex")  

  means <- matrix(
             c(weighted.mean(totoop(result$scf[[1]]), conData$weight),
               weighted.mean(result$totOop.std.nh, conData$weight),
               weighted.mean(result$totOop.std.nd, conData$weight),

               weighted.mean(result$totOop.nhstd.h, conData$weight),
               weighted.mean(totoop(result$scf[[2]]), conData$weight),
               NA,

               weighted.mean(result$totOop.ndstd.d, conData$weight),
               NA,
               weighted.mean(totoop(result$scf[[3]]), conData$weight),

               weighted.mean(totoop(result$SimJoinZero), conData$weight),
               weighted.mean(result$totOop.jz.nh, conData$weight),
               weighted.mean(result$totOop.jz.nd, conData$weight),

               weighted.mean(result$totOop.nh.h, conData$weight),
               weighted.mean(totoop(result$SimNoHole), conData$weight),
               NA,

               weighted.mean(result$totOop.nd.d, conData$weight),
               NA,
               weighted.mean(totoop(result$SimNoDed), conData$weight)
               ),
             ncol=3, byrow=TRUE)
  
  colnames(means) <- c("Baseline","Filled gap","Zero deductible")
  rownames(means) <- c("Std Ben","Std Ben-Filled gap","Std Ben-Zero Ded",
                       "Observed","Observed-Filled gap","Observed-Zero Ded")
  
  total <- c(weighted.mean(subset(stdB,t==T)$cumCost, conData$weight),
             weighted.mean(subset(stdBnh,t==T)$cumCost, conData$weight),
             weighted.mean(subset(dEqSnh,t==T)$cumCost, conData$weight),
             weighted.mean(subset(sim0,t==T)$cumCost, conData$weight),
             weighted.mean(subset(simNoHole,t==T)$cumCost, conData$weight),
             weighted.mean(subset(simNoDed,t==T)$cumCost, conData$weight)
             )
  
  insurer <- matrix(total,ncol=1) %*% matrix(rep(1,3),nrow=1) -means
  filename <- paste(outDir,"oopInsCost-behaveConst.txt",sep="")
  sink(filename)
  cat("\nOOP\n")
  print(means)
  cat("\nInsurer\n")
  print(insurer)
  sink()
             
  ## summary statistics of oop spending with and without hole
  filename <- paste(outDir,"stdInsCost.tex",sep="")
  tabd <-tab[1:nrow(tabo),]-tabo
  tabd$group <- tab$group[1:nrow(tabd)]
  print(xtable(tabd,align=rep("c",8),digits=0,display=c("s","s",rep("f",6))),
        file=filename,type="latex")  

  ## plot of budget sets
  p <- result$cf[[1]][c("xcut","ycut","slope")]
  xhi <- 20000
  yhi <- (xhi-p$xcut[4])*p$slope[4]+p$ycut[4]
  p <- result$cf[[2]][c("xcut","ycut","slope")]
  yhinh <- (xhi-p$xcut[4])*p$slope[4]+p$ycut[4]
  p <- result$cf[[3]][c("xcut","ycut","slope")]
  yhid <- (xhi-p$xcut[4])*p$slope[4]+p$ycut[4]

  fd <- rbind(data.frame(x=c(result$cf[[1]]$xcut,xhi),
                         y=c(result$cf[[1]]$ycut,yhi),
                         group="baseline"),
              data.frame(x=c(result$cf[[2]]$xcut,xhi),
                         y=c(result$cf[[2]]$ycut,yhinh),
                         group="filled gap"),
              data.frame(x=c(result$cf[[3]]$xcut,xhi),
                         y=c(result$cf[[3]]$ycut,yhid),
                         group="reduced deductible"))
  fitPlot <- ggplot(data=fd,aes(x=x,y=y,colour=group)) + themeLE() + 
    geom_line(alphe=0.7) +
      scale_y_continuous(name="out of pocket") +
        scale_x_continuous(name="spending") +
          scale_colour_manual(values=colors[2:4]) + 
            theme(legend.position = "bottom", legend.title=element_blank())
  printDev(paste(outDir,"stdPlan.pdf",sep=""),width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  figData <- c(figData, list(ggplot_build(fitPlot)$data[[1]]))
  names(figData)[length(figData)] <- "StandardBudget"


  ## histogram of spending change without hole
  maxc <- 5000
  minc <- -2000
  fd <-data.frame(x=(subset(stdBnh,t==T)$cumCost) - subset(stdB,t==T)$cumCost,
                  w=conData$weight)
  fd$x[fd$x>maxc] <- maxc
  fd$x[fd$x<minc] <- minc
  p0 <- weighted.mean(fd$x==0,fd$w)
  fd <- subset(fd, x!=0)
  fitPlot <- ggplot(data=fd, aes(x=x,y=..density..,fill="this does nothing",weight=w)) + themeLE() + 
    geom_histogram(alpha=0.7,position="dodge",binwidth=maxc/nbin) +
    scale_x_continuous(name="change in spending | change ≠ 0") +
    scale_fill_manual(values=colors[3],guide="none") +
    annotate("text",x=0,y=.0045,label=sprintf("P(change=0) = %.3f",p0), size=4, hjust=0) 
  printDev(paste(outDir,"std_nogapChange.pdf",sep=""),width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  figData <- c(figData, list(ggplot_build(fitPlot)$data[[1]]))
  names(figData)[length(figData)] <- "FilledGapChange.StdBenefit"

} else {
  warning(paste(filename,"exists, not recreating"))
}


## spending change as a function of initial spending: no hole, observed plans
filename <- paste(outDir,"obs_nhDspendVspend.pdf",sep="")
if (!file.exists(filename) || forceFig) {
  sourceCpp("kernreg.cpp")
  fd <-data.frame(dx=(subset(simNoHole,t==T)$cumCost - subset(sim0,t==T)$cumCost),
                  x=subset(sim0,t==T)$cumCost,w=conData$weight)
  step <- 200
  maxx <- 10000
  x <- seq(0,maxx,by=step)
  bw <- 500
  discrete <- data.frame(x=x,dx=kernreg(x,fd$x,fd$dx,bw,fd$w),
                         quant=kernquantreg(x,c(.1,.25,.5,.75,.9),fd$x,fd$dx,bw, fd$w))
  pd <- data.frame(x=result$cf[[1]]$xcut[2:4])
  fitPlot <- ggplot(data=discrete, aes(x=x)) + themeLE() + 
    ##geom_point(alpha=0.01) +
    #geom_smooth(se=FALSE) +
    #geom_smooth(method=lm,se=FALSE,formula=y ~ poly(x,4),color="red") +
    #geom_smooth(method=lm,se=FALSE,formula=y ~ poly(x,10),color="green") +
    geom_line(alpha=1,aes(y=dx,colour="mean")) +
    geom_line(alpha=1,aes(y=quant.1,colour="Q10")) + 
    geom_line(alpha=1,aes(y=quant.2,colour="Q25")) +
    geom_line(alpha=1,aes(y=quant.3,colour="Q50")) +
    geom_line(alpha=1,aes(y=quant.4,colour="Q75")) +
    geom_line(alpha=1,aes(y=quant.5,colour="Q90")) +
    geom_rug(sides="b",aes(x=x,y=NULL),data=pd,guide="none",color="black",alpha=0.5) +
    annotate("text",x=pd$x[1],y=-400,label="Deductible", size=4, hjust=0, vjust=0.5, angle=75,alpha=0.3) +
    annotate("text",x=pd$x[2],y=-400,label="ICL", size=4, hjust=0, vjust=0.5, angle=75,alpha=0.3) +
    annotate("text",x=pd$x[3],y=-400,label="CCL", size=4, hjust=0, vjust=0.5, angle=75,alpha=0.3) +
    scale_x_continuous(name="initial spending") +
    scale_y_continuous(name="change in spending") +
    scale_colour_manual(values=c("#000000",brewer.pal(5,"YlGnBu"))) +
    theme(legend.position = "bottom", legend.title=element_blank())
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  bp <-  ggplot_build(fitPlot)
  pd <- bp$data[[1]]
  names(pd)[names(pd)=="y"] <- "mean"
  pd$q10 <- bp$data[[2]]$y
  pd$q25 <- bp$data[[3]]$y
  pd$q50 <- bp$data[[4]]$y
  pd$q75 <- bp$data[[5]]$y
  pd$q90 <- bp$data[[6]]$y
  figData <- c(figData,list(pd))
  names(figData)[length(figData)] <- "changeFillGap"

  filename <- paste(outDir,"std_nhDspendVspend_ce.pdf",sep="")
  discrete$ex <- sapply(discrete$x, FUN = function(x)
                        { weighted.mean(fd$dx[fd$x<x], fd$w[fd$x<x]) })
  discrete$exg <- sapply(discrete$x, FUN = function(x)
                         { weighted.mean(fd$dx[fd$x>x], fd$w[fd$x>x]) })
  discrete$pless <- sapply(discrete$x, FUN = function(x) { weighted.mean(fd$x<x,fd$w) })
  fitPlot <- ggplot(data=discrete, aes(x=x)) + themeLE() + 
    geom_line(aes(y=ex, colour="E[change|initial<x]")) +  
    geom_line(aes(y=exg, colour="E[change|initial>x]"))
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  
  filename <- paste(outDir,"obs_nhDspendVspend_p.pdf",sep="")
  fitPlot <- ggplot(data=discrete, aes(x=x)) + themeLE() + 
    geom_line(aes(y=pless, colour="P[initial<x]"))
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()

  ## table
  filename <- paste(outDir,"obs_nhDspendVspend.tex",sep="")
  cat("\\begin{table}[ht] \\caption{Change in spending conditional on initial spending} \n
\\begin{center}\\begin{tabular}{cc cc cccc} \n
 &         &      &  & \\multicolumn{4}{c}{Quantiles} \\\\ \n
 & Portion & Mean & Std.Dev. & 0.25 & 0.5 & 0.75 & 0.9 \\\\ \n",file=filename,append=FALSE)

  grp <- fd$x<conData$icl
  cat(sprintf("Below ICL & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)

  grp <- fd$x>=conData$icl
  cat(sprintf("Above ICL & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
                            weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)

  grp <- fd$x<conData$ded
  cat(sprintf("Below deductible & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)

  grp <- fd$x>=conData$ded
  cat(sprintf("Above deductible & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)


  grp <- fd$x<conData$icl-200
  cat(sprintf("Below ICL-200 & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)
  
  grp <- abs(fd$x-conData$icl)<=200
  cat(sprintf("Within 200 of ICL & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)
  
  grp <- fd$x<1000
  cat(sprintf("Below \\$1000 & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)

  cat("\\\\ \n",file=filename,append=TRUE)

  cat("\\end{tabular}\\end{center}\\end{table}\n",file=filename,append=TRUE)
} else {
  warning(paste(filename,"exists, not recreating"))
}

## spending change as a function of initial spending: no hole 
filename <- paste(outDir,"std_nhDspendVspend.pdf",sep="")
if (!file.exists(filename) || forceFig) {
  sourceCpp("kernreg.cpp")
  fd <-data.frame(dx=(subset(stdBnh,t==T)$cumCost - subset(stdB,t==T)$cumCost),
                  x=subset(stdB,t==T)$cumCost,w=conData$weight)
  step <- 200
  maxx <- 10000
  x <- seq(0,maxx,by=step)
  bw <- 500
  discrete <- data.frame(x=x,dx=kernreg(x,fd$x,fd$dx,bw,fd$w),
                         quant=kernquantreg(x,c(.1,.25,.5,.75,.9),fd$x,fd$dx,bw, fd$w))
  pd <- data.frame(x=result$cf[[1]]$xcut[2:4])
  fitPlot <- ggplot(data=discrete, aes(x=x)) + themeLE() + 
    ##geom_point(alpha=0.01) +
    #geom_smooth(se=FALSE) +
    #geom_smooth(method=lm,se=FALSE,formula=y ~ poly(x,4),color="red") +
    #geom_smooth(method=lm,se=FALSE,formula=y ~ poly(x,10),color="green") +
    geom_line(alpha=1,aes(y=dx,colour="mean")) +
    geom_line(alpha=1,aes(y=quant.1,colour="Q10")) + 
    geom_line(alpha=1,aes(y=quant.2,colour="Q25")) +
    geom_line(alpha=1,aes(y=quant.3,colour="Q50")) +
    geom_line(alpha=1,aes(y=quant.4,colour="Q75")) +
    geom_line(alpha=1,aes(y=quant.5,colour="Q90")) +
    geom_rug(sides="b",aes(x=x,y=NULL),data=pd,guide="none",color="black",alpha=0.5) +
    annotate("text",x=pd$x[1],y=-400,label="Deductible", size=4, hjust=0, vjust=0.5, angle=75,alpha=0.3) +
    annotate("text",x=pd$x[2],y=-400,label="ICL", size=4, hjust=0, vjust=0.5, angle=75,alpha=0.3) +
    annotate("text",x=pd$x[3],y=-400,label="CCL", size=4, hjust=0, vjust=0.5, angle=75,alpha=0.3) +
    scale_x_continuous(name="initial spending") +
    scale_y_continuous(name="change in spending") +
    scale_colour_manual(values=c("#000000",brewer.pal(5,"YlGnBu"))) +
    theme(legend.position = "bottom", legend.title=element_blank())
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  ## save plot data to create xls file for Liran :(
  bp <-  ggplot_build(fitPlot)
  pd <- bp$data[[1]]
  names(pd)[names(pd)=="y"] <- "mean"
  pd$q10 <- bp$data[[2]]$y
  pd$q25 <- bp$data[[3]]$y
  pd$q50 <- bp$data[[4]]$y
  pd$q75 <- bp$data[[5]]$y
  pd$q90 <- bp$data[[6]]$y
  figData <- c(figData,list(pd))
  names(figData)[length(figData)] <- "changeFillGap"

  filename <- paste(outDir,"std_nhDspendVspend_ce.pdf",sep="")
  discrete$ex <- sapply(discrete$x, FUN = function(x)
                        { weighted.mean(fd$dx[fd$x<x], fd$w[fd$x<x]) })
  discrete$exg <- sapply(discrete$x, FUN = function(x)
                         { weighted.mean(fd$dx[fd$x>x], fd$w[fd$x>x]) })
  discrete$pless <- sapply(discrete$x, FUN = function(x) { weighted.mean(fd$x<x,fd$w) })
  fitPlot <- ggplot(data=discrete, aes(x=x)) + themeLE() + 
    geom_line(aes(y=ex, colour="E[change|initial<x]")) +  
    geom_line(aes(y=exg, colour="E[change|initial>x]"))
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()
  
  filename <- paste(outDir,"std_nhDspendVspend_p.pdf",sep="")
  fitPlot <- ggplot(data=discrete, aes(x=x)) + themeLE() + 
    geom_line(aes(y=pless, colour="P[initial<x]"))
  printDev(filename,width=fdims[1],height=fdims[2])
  print(fitPlot)
  dev.off()

  
  

  ## table
  pd <- data.frame(x=result$cf[[1]]$xcut[2:4])
  filename <- paste(outDir,"std_nhDspendVspend.tex",sep="")
  cat("\\begin{table}[ht] \\caption{Change in spending conditional on initial spending} \n
\\begin{center}\\begin{tabular}{cc cc cccc} \n
 &         &      &  & \\multicolumn{4}{c}{Quantiles} \\\\ \n
 & Portion & Mean & Std.Dev. & 0.25 & 0.5 & 0.75 & 0.9 \\\\ \n",file=filename,append=FALSE)

  grp <- fd$x<pd$x[2]
  cat(sprintf("Below ICL & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)

  grp <- fd$x>=pd$x[2]
  cat(sprintf("Above ICL & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
                            weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)

  grp <- fd$x<pd$x[1]
  cat(sprintf("Below deductible & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)

  grp <- fd$x>=pd$x[1]
  cat(sprintf("Above deductible & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)


    grp <- fd$x<pd$x[2]-200
  cat(sprintf("Below ICL-200 & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)
  
  grp <- abs(fd$x-pd$x[2])<=200
  cat(sprintf("Within 200 of ICL & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)
  cat("\\\\ \n",file=filename,append=TRUE)
  
  grp <- fd$x<1000
  cat(sprintf("Below \\$1000 & %.2f & %.0f & %.0f",
              weighted.mean(grp,fd$w),
              weighted.mean(fd$dx[grp],fd$w[grp]),
              weighted.sd(fd$dx[grp],fd$w[grp]))
      , file=filename,append=TRUE)
  cat(sprintf(" & %.0f",
              weighted.quantile(fd$dx[grp],probs=c(.25,.5,.75,.9),fd$w[grp]))
      , file=filename,append=TRUE)

  cat("\\\\ \n",file=filename,append=TRUE)

  cat("\\end{tabular}\\end{center}\\end{table}\n",file=filename,append=TRUE)
} else {
  warning(paste(filename,"exists, not recreating"))
}


## Pr(type|near icl)
icldiff <- subset(stdB,t==T)$cumCost - dic[1,2]
type <- factor(result$rdraw$type)
weighted.factor.P <- function(type,w) {
  sapply(levels(type),FUN=function(x) { weighted.mean(type==x,w) } )
}
filename <- paste(outDir,"typeNearICL.tex",sep="")
cat("\\begin{table}[ht] \\caption{Types conditional on spending} \n
\\begin{center}\\begin{tabular}{c cccccc} \n
 & \\multicolumn{5}{c}{Type} \\\\ \n
 & 1 & 2 & 3 & 4 & 5 \\\\ \n",file=filename,append=FALSE)
cat("$\\mathrm{P}(type)$",
    sprintf("%.3f",weighted.factor.P(type,conData$weight)[ord]),
    sep=" & ",file=filename,append=TRUE);
cat(" \\\\ \n",file=filename,append=TRUE)
g <- icldiff< (-200)
cat("$\\mathrm{P}(type | spend-icl < -200)$",    
    sprintf("%.3f",weighted.factor.P(type[g],conData$weight[g])[ord]),
    sep=" & ",file=filename,append=TRUE);
cat(" \\\\ \n",file=filename,append=TRUE)
g <- abs(icldiff)<200
cat("$\\mathrm{P}(type | |spend-icl|<200)$",
    sprintf("%.3f",weighted.factor.P(type[g],conData$weight[g])[ord]),
    sep=" & ",file=filename,append=TRUE);
cat(" \\\\ \n",file=filename,append=TRUE)
g <- icldiff>200
cat("$\\mathrm{P}(type | spend-icl>200)$",
    sprintf("%.3f",weighted.factor.P(type[g],conData$weight[g])[ord]),
    sep=" & ",file=filename,append=TRUE);
cat(" \\\\ \n",file=filename,append=TRUE)
cat("\\end{tabular}\\end{center}\\end{table}\n",file=filename,append=TRUE)  


## spending as a function of risk score
filename <- paste(outDir,"spendRS-1.pdf",sep="")
if (NX>1 && (!file.exists(filename) || forceFig)) {
  bins <- 4
  afd <-data.frame(y=subset(costData,t==T)$cumCost,
                  ys=subset(simCost,t==T)$cumCost,
                  x=conData$rs,w=conData$weight)
  afd$x[conData$age==65] <- NA
  afd <- subset(afd,conData$age>65)
  cutoffs <- quantile(afd$x, probs=seq(0,1,length.out=bins+1),na.rm=TRUE)
  afd$group <- sapply(afd$x,function(x) { sum(x<=cutoffs)} )
  afd$group[afd$group>bins] <- bins ## minimal x will have group=bins+1

  for(b in 1:bins) {
    fd <- rbind(data.frame(x=subset(afd,group==b)$y,group="observed",w=subset(afd,group==b)$w),
                data.frame(x=subset(afd,group==b)$ys,group="estimated",w=subset(afd,group==b)$w))
    maxc <- 6000 
    tab <- data.table(fd)[,list(p0=weighted.mean(x==0,w),
                                mean=weighted.mean(x,w),
                                sd=weighted.sd(x,w),
                                median=weighted.median(x,w)),by=group]
    fd$x[fd$x>maxc] <- maxc
    fitPlot <- ggplot(data=fd, aes(x=x,y=2*..count../sum(..count..),fill=group,weight=w)) + themeLE() + 
      stat_bin(breaks=seq(0.0,maxc+1,by=100),alpha=0.3,position="dodge") + 
      scale_x_continuous(name="total spending") +
      scale_y_continuous(name="portion") + 
      scale_fill_manual(values=colors) + 
      theme(legend.position = "bottom",legend.title=element_blank()) +
      annotate("text",x=500,y=.06,
               label=sprintf(" observed: P(0)=%.3f median=%.2f, mean=%.2f, and sd=%.2f",
                 tab$p0[1],tab$median[1],tab$mean[1],tab$sd[1]), size=4, hjust=0,color=colors[1]) + 
      annotate("text",x=500,y=.05,
               label=sprintf("estimated: P(0)=%.3f median=%.2f, mean=%.2f, and sd=%.2f",
               tab$p0[2],tab$median[2],tab$mean[2],tab$sd[2]), size=4, hjust=0,color=colors[2]) 
    filename <- paste(outDir,sprintf("spendRS-%d.pdf",b),sep="")
    printDev(filename,width=fdims[1],height=fdims[2])
    print(fitPlot)
    dev.off()
    ## save plot data to create xls file for Liran :(
    figData <- c(figData, list(ggplot_build(fitPlot)$data[[1]]))
    names(figData)[length(figData)] <- sprintf("Fit.Histogram.riskScoreQuartile%d",b)
  }
} else {
  warning(paste(filename,"exists, not recreating"))
}



if (forceFig) {
  ## Required perl module is now missing :(  
  #require(WriteXLS)
  #WriteXLS("figData","medDFigures.xls")
  save(figData,file="medDFigures.Rdata")
}

T <- 51
delay <- (matrix(result$Simulated[[1]]$delay,nrow=(T+1),ncol=N))
rowMeans(delay)
mean(result$Simulated[[1]]$totalDelay)
rowMeans(delay) %*% (pm$deltaTheta^((T+1):1))
spend.obs <- matrix(result$Observed$cost,nrow=(T+1),ncol=N)
rowMeans(spend.obs)
spend <- matrix(result$Simulated[[1]]$cost,nrow=(T+1),ncol=N)
rowMeans(spend)
theta <- matrix(fullIns$cost,nrow=(T+1),ncol=N)
rowMeans(theta)
never <- theta-delay-spend
rowMeans(never) %*% (pm$deltaTheta^((T+1):1))
#spend.obs <- array(result$Observed$cost, dim=c((T+1),3,
df <- data.frame(week=0:51,theta=rowMeans(theta),treated=rowMeans(spend), delayed=rowMeans(delay),
                 spend.obs=rowMeans(spend.obs))
fig <- ggplot(data=df, aes(x=week)) + themeLE() + 
  geom_line(alpha=1,aes(y=treated/theta,colour="treated")) +
  geom_line(alpha=1,aes(y=delayed/theta,colour="delayed")) + 
  geom_line(alpha=1,aes(y=(theta-treated-delayed)/theta,colour="never treated")) +
  scale_x_continuous(name="week") +
  scale_y_continuous(name="portion of theta") +
  scale_colour_manual(values=colors[2:4]) + 
  theme(legend.position = "bottom", legend.title=element_blank())
filename <- paste(outDir,"shareDelayed.pdf",sep="")
printDev(filename,width=fdims[1],height=fdims[2])
print(fig)
dev.off()
figData <- c(figData, list(ggplot_build(fig)$data[[1]]))
names(figData)[length(figData)] <- "shareDelayed"  


fig <- ggplot(data=df, aes(x=week)) + themeLE() + 
  geom_line(alpha=1,aes(y=spend.obs),linetype=1) +
  geom_line(alpha=1,aes(y=treated),linetype=2) + 
  scale_x_continuous(name="week") +
  scale_y_continuous(name="Average spending") +
  scale_colour_manual(values=colors[2:4]) + 
  theme(legend.position = "bottom", legend.title=element_blank())
filename <- paste(outDir,"spendWeek.pdf",sep="")
printDev(filename,width=fdims[1],height=fdims[2])
print(fig)
dev.off()
figData <- c(figData, list(ggplot_build(fig)$data[[1]]))
names(figData)[length(figData)] <- "spendWeek"




df <- data.frame(week=0:51, 
                 spend.stdb=rowMeans(matrix(result$scf[[1]][[1]]$cost,nrow=(T+1),ncol=N)),
                 spend.stdb.nh=rowMeans(matrix(result$scf[[2]][[1]]$cost,nrow=(T+1),ncol=N))
                 )
fig <- ggplot(data=df, aes(x=week)) + themeLE() + 
  geom_line(alpha=1,aes(y=spend.stdb,colour="spending with standard benefit")) +
  geom_line(alpha=1,aes(y=spend.stdb.nh,colour="spending without gap")) + 
  scale_x_continuous(name="week") +
  scale_y_continuous(name="average spending") +
  scale_colour_manual(values=colors[2:3]) + 
  theme(legend.position = "bottom", legend.title=element_blank())
filename <- paste(outDir,"fillGap-timing.pdf",sep="")
printDev(filename,width=fdims[1],height=fdims[2])
print(fig)
dev.off()
figData <- c(figData, list(df))
names(figData)[length(figData)] <- "fillGap.timing"
             

id <- result$Observed$id+1
year <- result$Observed$year+1
cc.obs <- array(dim=c(T+1, max(year), max(id)))
cc.est <- cc.obs
obs <- matrix(result$Observed$cumCost,nrow=T+1,ncol=N)
est <- matrix(result$Simulated[[1]]$cumCost,nrow=T+1,ncol=N)
for (t in 1:(T+1)) {
  cc.obs[cbind(t,year,id)] <- obs[t,]
  cc.est[cbind(t,year,id)] <- est[t,]
}
excessJan <- function(cc) {
  cc[4,,]/(cc[52,,]-cc[4,,]+1e-4)
}
icl <- matrix(nrow=max(year),ncol=max(id))
totalDelay <- icl
icl[cbind(year,id)] <- conData$icl
totalDelay[cbind(year,id)] <- result$Simulated[[1]]$totalDelay
cond.mean <- function(xj,cc,icl,mid,width) {
  in.bin <- abs( (cc[T+1,,]-icl)-mid)<width
  n <- 0
  mu <- 0
  for(y in 2:max(year)) {
    n <- n+sum(!is.na(xj[y,in.bin[y-1,]]))
    mu <- mu+sum(xj[y,in.bin[y-1,]],na.rm=TRUE)
  }
  return(mu/n)
}

bw <- 50
bins <- seq(-2200,2000,by=2*bw)
w.lo <- 8
w.hi <- 26
df <- data.frame(icl.diff=bins,
                 excess.obs=sapply(bins,function(x) {
                   #cond.mean(excessJan(cc.obs),cc.obs,icl,x,bw) }),
                   cond.mean(cc.obs[4,,],cc.obs,icl,x,bw)/
                     cond.mean(cc.obs[w.hi,,]-cc.obs[w.lo,,],cc.obs,icl,x,bw) * (w.hi-w.lo)/4
                 }), 
                 excess.est=sapply(bins,function(x) {                   
                   #cond.mean(excessJan(cc.est),cc.est,icl,x,bw) }))
                   cond.mean(cc.est[4,,]+totalDelay,cc.est,icl,x,bw)/
                     cond.mean(cc.est[w.hi,,]-cc.est[w.lo,,],cc.est,icl,x,bw) * (w.hi-w.lo)/4
                 })
                 )
             
est.pre <- cond.mean(cc.est[4,,]+totalDelay,cc.est,icl,-1250,750)/
  cond.mean(cc.est[w.hi,,]-cc.est[w.lo,,],cc.est,icl,-1250,750) * (w.hi-w.lo)/4

obs.pre <- cond.mean(cc.obs[4,,],cc.obs,icl,-1250,750)/
  cond.mean(cc.obs[w.hi,,]-cc.obs[w.lo,,],cc.obs,icl,-1250,750) * (w.hi-w.lo)/4

est.kink <- cond.mean(cc.est[4,,]+totalDelay,cc.est,icl,250,250)/
  cond.mean(cc.est[w.hi,,]-cc.est[w.lo,,],cc.est,icl,250,250) * (w.hi-w.lo)/4

obs.kink <- cond.mean(cc.obs[4,,],cc.obs,icl,250,250)/
  cond.mean(cc.obs[w.hi,,]-cc.obs[w.lo,,],cc.obs,icl,250,250) * (w.hi-w.lo)/4

fig <- ggplot(data=df,aes(x=icl.diff)) + themeLE() +
  geom_line(aes(y=excess.obs),linetype=1) +
  geom_line(aes(y=excess.est),linetype=2) +
  geom_segment(aes(x=-2000,y=obs.pre,yend=obs.pre,xend=-500),linetype=1,colour=colors[2]) +
  geom_segment(aes(x=-2000,y=est.pre,yend=est.pre,xend=-500),linetype=2,colour=colors[2]) +
  geom_segment(aes(x=0,y=obs.kink,yend=obs.kink,xend=500),linetype=1,colour=colors[2]) +
  geom_segment(aes(x=0,y=est.kink,yend=est.kink,xend=500),linetype=2,colour=colors[2]) +  
  scale_x_continuous(name="ICL-spending in previous year") +
  scale_y_continuous(name="E[Jan spending | ICL-spending]/E[Feb-Dec spending | ICL - spending]") +
  scale_colour_manual(values=colors[2:4]) + 
  theme(legend.position = "bottom", legend.title=element_blank())
filename <- paste(outDir,"excessJanuary.pdf",sep="")
printDev(filename,width=fdims[1],height=fdims[2])
print(fig)
dev.off()

figData <- c(figData, list(ggplot_build(fig)$data[[1]]))
names(figData)[length(figData)] <- "excessJanuary"


save(figData,file=paste(outDir,"figures.Rdata",sep=""))

source("createXLS.R")
createXLS(outDir)

## create slides
tokens <- strsplit(outDir,"/",1)
filename <- paste(tokens[[1]][length(tokens[[1]])],".pdf",sep="")
system(sprintf("cd ../../tex/slides && make -B %s",filename))
