***************************************************************************************************;
* MAKE.SAS
* This file calls on macros to make the four crosswalks
*   BENE_ID - EHIC 
*   PHYS_ID - SPECIALTY
***************************************************************************************************;

** ALL DIRECTORIES LIVE HERE -- READS IN DIRECTORIES for the 20 pct Extract;
** We use the 20 pct extract because that encompasses more claims than the 05 pct;

*Data libraries;
%macro setlibs;
	%do year = 1998 %to 2010;
		libname car&year. "/disk/agedisk2/medicare/data/20pct/car/&year.";
	%end;
%mend;
%setlibs;

libname tempDir "/disk/agedisk3/medicare.work/finkels-DUA22559/tangb/contract_design/code/RiskScoreInputs";
libname outDir "/disk/agedisk3/medicare.work/finkels-DUA22559/tangb/contract_design/code/RiskScoreOutputs/MedicarePartsAandB";
%include "/disk/agedisk3/medicare.work/finkels-DUA22559/tangb/contract_design/code/CalculatingRiskScoresAndUtilization/UtilityMacros.sas";

%macro stack(filepfx=, low=, high=);
	%do index=&low %to &high;
	  &filepfx&index
	%end;
%mend stack;

%macro stackcar;
	data car1998;
		set %stack(filepfx=car1998.car20h_1998_,low=1,high=12)
			%stack(filepfx=car1998.car20i_1998_,low=1,high=71);
	run;
	data car1999;
		set %stack(filepfx=car1999.car20h_1999_,low=1,high=12)
			%stack(filepfx=car1999.car20i_1999_,low=1,high=70);
	run;
	data car2000;
		set %stack(filepfx=car2000.car20i_2000_,low=1,high=97);
	run;
	data car2001;
		set %stack(filepfx=car2001.car20i_2001_,low=1,high=100);
	run;
	data car2002;
		set %stack(filepfx=car2002.car20i_2002_lnits,low=1,high=21);
	run;
	data car2003;
		set %stack(filepfx=car2003.car20i_2003_lnits,low=1,high=16);
	run;
	data car2004;
		set %stack(filepfx=car2004.car20i_2004_lnits,low=1,high=24);
	run;
	data car2005;
		set %stack(filepfx=car2005.car20i_2005_lnits,low=1,high=24);
	run;
	data car2006;
		set %stack(filepfx=car2006.car20_2006_lnits,low=1,high=24);
	run;
	data car2007;
		set %stack(filepfx=car2007.car20_2007_lnits,low=1,high=100);
	run;
	data car2008;
		set %stack(filepfx=car2008.car20_2008_lnits,low=1,high=100);
	run;
	data car2009;
		set %stack(filepfx=car2009.carl_2009_,low=1,high=100);
	run;
	data car2010;
		set car2010.carl2010;
	run;
%mend;
%stackcar;

** Run Macro to Generate Specialty Lists;
%macro GenUpinNpiSpcl(startyear=, endyear=);

    %do year = &startyear. %to &endyear.;

        %if &year.>=2006 %then %do;
            %let physid=prf_upin prf_npi;
        %end;
        %if &year.<=2005 %then %do;
            %let physid=prf_upin; 
        %end;

        data tempDir.spcl&year.;
            set car&year.;
            keep &physid. hcfaspcl; 
        run;

        proc sort data=tempDir.spcl&year. nodups; by _all_; run;
    %end;

    data spcllist;
        set %RepeatN(pfx=tempDir.spcl, Nlow=&startyear., Nhigh=&endyear.);
		length prf_npi $12;
		length prf_upin $12;

    run; 

    data spcllist;
        set spcllist;
        rename prf_upin = at_upin;
		rename prf_npi = at_npi;
        rename hcfaspcl=spcl;
    run; 

    data outDir.spcllist;
        set spcllist;
        keep at_upin at_npi spcl;
    run;
	
%mend;
%GenUpinNpiSpcl(startyear=1998, endyear=2010);


 
